#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "fileutil.h"
#include "vm2.h"
#include "ran1.h"

extern void  Calc_P();
extern void  Est_capture_probs();
extern void  Ecp_random_model();
extern void  Get_h();
extern void  Get_p1r();
extern float Get_lh();

float  ranges[50], mut_range;
extern long idnum;
extern float eta, mu, zeta, delta, beta, min_area, B;
extern int   dayc, init_ps, ntprns;
extern char  z_eq_b, use_edge, do_sa, point_fn[];
extern struct m_day *days;
FILE *pt_file;

/********************** Error **************************/
float Get_error()
{
  float err;
  int loop;
  time_t t,t2;

#if 1
    Calc_P();
    Ecp_random_model();
    Get_p1r();
    Get_h();
    err=-Get_lh();
//    printf("Err: %f\n", err);
//    exit(1);
#endif

#if 0
  t=time(NULL);
  for(loop=1;loop<500;loop++)
    Calc_P();
  t2=time(NULL);
  printf("TP: %ld\n", t2-t);

  for(loop=1;loop<500;loop++)
    Ecp_random_model();
  t2=time(NULL);
  printf("TE: %ld\n", t2-t);

  for(loop=1;loop<500;loop++)
    Get_p1r();
  t2=time(NULL);
  printf("TR: %ld\n", t2-t);

  for(loop=1;loop<500;loop++)
    Get_h();
  t2=time(NULL);
  printf("TH: %ld\n", t2-t);

  for(loop=1;loop<500;loop++)
    {
      printf("*");
      err=-Get_lh();
    }
  t2=time(NULL);
  printf("TE: %ld\n", t2-t);

  exit(1);
#endif
  return err;
}


/************* Rap_point *************************************/
void Rap_points(float *pars, float fit)
{
  if (fit>=10000)
    return;

  fprintf(pt_file,"%-10.4f",fit);
  fprintf(pt_file,"%-10.4f",pars[1]);
  fprintf(pt_file,"%-10.4f",pars[2]);
  fprintf(pt_file,"%-10.4f",pars[3]);
  fprintf(pt_file,"%-10.4f",pars[4]);
  fprintf(pt_file,"%-10.4f",pars[5]);
  if (z_eq_b!=Yes)
    fprintf(pt_file,"%-10.4f",pars[6]);
  fprintf(pt_file,"\n");
}



/********************** Decode ***************************************/
void Decode(float *str)
{

  if (str[1]<=0)
    str[1]=0.0000001;
  if (str[1]>=1)
    str[1]=0.9999999;
  mu = str[1];

  if (str[3]>0)
    str[3]=0;
  zeta = str[3];

  if (str[2]<0)
    str[2]=0;
  if (str[2]>=(1.0/pow(min_area, zeta)))
    str[2]= 0.9999999/pow(min_area, zeta);
  eta= str[2];

  if (str[4]<0)
    str[4]=0;
  beta = str[4];

  if (str[5]<0)
    str[5]=0;
  delta = str[5];

  if (z_eq_b==Yes)
    B=-zeta;
  else
    {
      if (str[6]<0)
	str[6]=0;
      B = str[6];
    }


}


/********************** Print_func ***********************************/
void Print_func(int generation, float estimate, float *str)
{
  Decode(str);
  printf("%i Err:%-7.4f :%-7.3f eta:%-7.3f zeta:%-7.3f :%-7.3f :%-7.3f B:%-7.3f\n",
	 generation, estimate, mu, eta, zeta, beta, delta, B);
}


/********************** Fitness **************************************/
float Fitness(float *str) {
  Decode(str);
  return Get_error();
}


/***************** Rand_str ***********************************************/
void Rand_str(float *str)
{
  int   loop;

  str[1]=0.3*ran1(&idnum);
  str[2]=0.7*ran1(&idnum);
  str[3]=-0.7*ran1(&idnum);
  str[4]=8.0*ran1(&idnum);
  str[5]=4.0*ran1(&idnum);
  if (z_eq_b==No)
    str[6]=0.7*ran1(&idnum);
}

/****************** Mutate_old *******************************************/
static void Mutate_old(float *str, float pm, int len)
{
  int loop;

  for(loop=1; loop<=len; loop++)
    {
      if (ran1(&idnum)<pm)
      	str[loop] += ranges[loop]*mut_range*gasdev(&idnum);
    }
}

void popcpy(float *dest, float *src, int len)
{
  memcpy((void *)dest, (void *)src, len*sizeof(float));
}

/************************* Get_start ***********************************/
void Get_start(float *pop, float *bf, float *bp, int str_len, int pop_size)
{
  int   loop;
  float seed[50], fit;

  printf("Initialization with %i evaluations\n", pop_size);
  for(loop=0; loop<pop_size; loop++)
    {
      Rand_str(seed);
      fit = Fitness(seed);
      Rap_points(seed,fit);
      if (fit<*bf)
	      {
		      popcpy(&pop[1], &seed[1], str_len);
		      *bf = fit;
		      popcpy(&bp[1], &seed[1], str_len);
	      }

    }

  printf("Starting point, fitn.: %0.0f %0.0f\n", *bf, Fitness(pop));
}

/********************** sa *********************************************/
void sa(float *cp, float *bp, float *bf, float temptr, int iter, int len)
{
  int   loop;
  float fit, cf, ocp[50];

  cf = Fitness(cp);
  Rap_points(cp,cf);
  printf("%0.1f ", cf);
  popcpy(&ocp[1], &cp[1], len);

  for (loop=0; loop<iter; loop++)
    {
      Mutate_old(cp, 1.0/len, len);
      fit = Fitness(cp);
      Rap_points(cp,fit);
      if (fit==10000)
	      {
		      popcpy(&cp[1], &ocp[1], len);
		      continue;
	      }
      if (fit<cf)
	      {
		      cf = fit;
		      if (fit< *bf)
		        {
		          *bf=fit;
		          popcpy(&bp[1], &cp[1], len);
		        }
		      popcpy(&ocp[1], &cp[1], len);
	      }
      else if (ran1(&idnum) < exp(-(fit-cf)/temptr))
	      {
		      cf = fit;
		      popcpy(&ocp[1], &cp[1], len);
	      }
      else
	      popcpy(&cp[1], &ocp[1], len);
    }
}


/****************** Estimate ****************************************/
int Estimate(int gens)
{
  int   loop, new_sin, c, PS, iter, jiter, iiter=100,j, L;
  long  nit;
  float point, pm, pc, temptr, ftol, pbf, bf, best_f,
       	T0,K,cp[50], bp[50], pbl;
  float orig_mu, orig_eta, orig_zeta, orig_B, orig_beta, orig_delta;

  Raport("PARAMETER ESTIMATION USING SA, %ld EVALUATIONS\n",(long)gens*100L);

  if (z_eq_b==Yes)
    L=5;
  else
    {
      L=6;
      ranges[6]=0.15;
    }

  pt_file=Open_file(point_fn, "w+t", "point ", Terminate_on_error);


  ranges[1]=0.1;
  ranges[2]=0.15;
  ranges[3]=0.2;
  ranges[4]=1.0;
  ranges[5]=1.0;

  bf = 100000000;
  Get_start(cp, &bf, bp, L, init_ps);
  pbf = bf;

  pbl = temptr = T0 = 1.0;
  K   = 100.0*gens;
  printf("\nStarting SA %0.3f \n", pbf);
  nit = 0;
  for (jiter=1;jiter<=gens;jiter++)
    {
      mut_range = exp(-jiter/(0.4*gens));
      printf("n");
      iter=iiter;
      temptr = T0*pow(1.0-nit/K, 2.0);
      sa(cp, bp, &bf, temptr, iter, L);
      if (bf < pbf)
	      {
		      pbf=bf;
		      printf("\nBest:%i %0.6f %0.1f ",jiter, temptr, bf);
		      pbl = temptr;
		      Print_func(jiter, bf, bp);
	      }
//      temptr *= 0.9;
      if ((temptr/pbl) < 0.75)
	     {
	       popcpy(&cp[1], &bp[1], L);
	     }
      nit += iiter;
    }

  best_f = Fitness(bp);
  printf("\nBest found %f \n", best_f);
  Print_func(jiter, best_f, bp);

  Raport("\nFINAL VALUES : \n");
  Raport("*** Err:%-7.4f :%-7.3f eta:%-7.3f zeta:%-7.3f :%-7.3f :%-7.3f B:%-7.3f \n",
      	 best_f, mu, eta, zeta, beta, delta, B);
  Raport("Parameter 1, mortality in patch                   = %0.4f\n", mu);
  Raport("Parameter 2, miration rate for 1 hectare patch    = %0.4f\n", eta);
  Raport("Parameter 3, scaling of migration with patch size = %0.4f\n", zeta);
  Raport("Parameter 4, distance-dependence in migration     = %0.4f\n", beta);
  Raport("Parameter 5, scales mortality during migration    = %0.4f\n", delta);
  Raport("Parameter 6, scaling of immigration into patch    = %0.4f\n\n", B);

  best_f = Fitness(bp);
  fprintf(pt_file,"****************************\n");
  fprintf(pt_file,"MIN ERROR  %-10.4f\n",best_f);
  fprintf(pt_file,"PARAMETERS:\n");
  fprintf(pt_file,"%-10.4f 1:mu\n",  mu);
  fprintf(pt_file,"%-10.4f 2:eta\n", eta);
  fprintf(pt_file,"%-10.4f 3:zeta\n",zeta);
  fprintf(pt_file,"%-10.4f 4:beta\n",beta);
  fprintf(pt_file,"%-10.4f 5:delta\n",delta);
  if (z_eq_b!=Yes)
    fprintf(pt_file,"%-10.4f 6:B\n",B);
  fprintf(pt_file,"****************************\n");

  fclose(pt_file);


  if (do_sa==No)
    return True;

  Raport("\n\nSensitivity analysis; \n");
  orig_mu = mu;
  for(mu=0; mu<=min(2.0*orig_mu,1.0); mu+=0.01)
    Raport("%-4.2f  %-7.4f\n", mu, Get_error());
  mu = orig_mu;

  Raport("\n\nSensitivity analysis; eta\n");
  orig_eta = eta;
  for(eta=0; eta<=min(2.0*orig_eta,1.0); eta+=0.01)
    Raport("%-4.2f  %-7.4f\n", eta, Get_error());
  eta = orig_eta;

  if (z_eq_b==No)
    {
      Raport("\n\nSensitivity analysis; zeta\n");
      orig_zeta = zeta;
      for(zeta=0; zeta>=2.0*orig_zeta; zeta-=0.01)
        {
          if (eta*pow(min_area, zeta)>1)
            break;
      	  Raport("%-4.2f  %-7.4f\n", zeta, Get_error());
        }
      zeta = orig_zeta;

      Raport("\n\nSensitivity analysis; B\n");
      orig_B = B;
      for(B=0.5*orig_B; B<=2.0*orig_B; B+=1.5*orig_B/50.0)
      	Raport("%-4.2f  %-7.4f\n", B, Get_error());
      B = orig_B;
    }
  else
    {
      Raport("\n\nSensitivity analysis; zeta & B\n");
      orig_zeta = zeta;
      for(zeta=0; zeta>=2.0*orig_zeta; zeta-=0.01)
	      {
          if (eta*pow(min_area, zeta)>1)
            break;
	        B = -zeta;
	        Raport("%-4.2f  %-7.4f\n", zeta, Get_error());
	      }
      zeta = orig_zeta;
      B = -orig_zeta;
    }

  Raport("\n\nSensitivity analysis; beta\n");
  orig_beta = beta;
  for(beta=0.5*orig_beta; beta<=2.0*orig_beta; beta+=1.5*orig_beta/50.0)
    Raport("%-5.2f  %-7.4f\n", beta, Get_error());
  beta = orig_beta;

  Raport("\n\nSensitivity analysis; delta\n");
  orig_delta = delta;
  for(delta=0.0; delta<=2.0*orig_delta; delta+=2.0*orig_delta/50.0)
    Raport("%-5.3f  %-7.4f\n", delta, Get_error());
  delta = orig_delta;

  ntprns = True;

  return True;
}
