/*
Virtual migration model parameterization.
Ilkka Hanski, Juha Alho and Atte Moilanen.
Numerical implementation by Atte Moilanen 18.03.1997 ->.
*/

#include <stdio.h>
#include <conio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include <stdarg.h>

#include "ran1.h"
#include "kbio.h"
#include "nrutils.h"
#include "fileutil.h"
#include "vmcl2.h"
#include "avevar.h"

#define ALPHAVER2 0


extern void  Close_rap_file();
extern float Estimate(int gens, float min_req_err);
extern float Estimate2(long f_evals, float prev, float *err, float max_err, float l_or_u);
extern float Get_error();
extern float Get_error_old();

long   idnum;
int    prints=True, rap_mode=True, pri=False;
char   **set_dat;
int    set_rc;

char   data_file_name[80],patch_file_name[80],
       days_file_name[80], output_file_name[80],
       sf_name[80], z_eq_b, use_edge, do_sa, div_or_bin;

int    pc, bfc, dayc, mdc, max_surv_int, init_ps, est_pat1, ntprns;
size_t matr_size;

long   f_evals;

float  **dist_mat, **P, **H, **H2, **H3, **H4, **pim, min_area, **Q;
float  **P_vec[365], orig_fit, cap_pm;
float  eta,  orig_eta;
float  mu,   orig_mu;
float  zeta, orig_zeta;
float  beta, orig_beta;
float  delta,orig_delta;
float  B,    orig_B;
int    **QCM, **obs_mat, **live_mat, fixed_vec[10], clfp, **marked;
float  obs_mean1, obs_mean2, obs_std;

struct patch     *patches;
struct m_day     *days;
struct butterfly *bflies;


/****************** Mat_cpy **********************************/
static near void Mat_cpy(float **m1, float **m2, int cols)
{
#if 0
  int loop,loop2;
  size_t clen;
  clen = (cols+1)*sizeof(float);
  for(loop=0; loop<=cols; loop++)
    memcpy(m2[loop], m1[loop], clen);
//    memcpy(&m2[loop][0], &m1[loop][0], clen);
#else
  memcpy(m2[0], m1[0], matr_size);
#endif
}

static near void Diag(float **m1, int rows)
{
  int loop, loop2;

  for(loop=1; loop<=rows; loop++)
    for(loop2=0; loop2<=loop; loop2++)
      m1[loop][loop2]=m1[loop2][loop]=0.0;

  for(loop=0; loop<=rows; loop++)
        m1[loop][loop]=1.0;
}

static near void Mat_multip(float **dest, float **m1, float **m2, int rows)
{
  int   loop, loop2, loop3;
  float *m1r, tmp;

  for(loop=0; loop<=rows; loop++)
    {
      m1r = m1[loop];
      for(loop2=0; loop2<=rows; loop2++)
        {
          tmp = 0.0;
          for(loop3=0; loop3<=rows; loop3++)
            tmp += m1r[loop3]*m2[loop3][loop2];

          dest[loop][loop2]=tmp;
        }
    }
#if 0
  for(loop=0; loop<=rows; loop++)
    {
      m1r = m1[loop];
      for(loop2=0; loop2<=rows; loop2++)
        {
          tmp = 0.0;
          elem2 = &m2[0][loop2];
          for(loop3=0; loop3<=rows; loop3++)
            {
              tmp += m1r[loop3]*(*elem2);//m2[loop3][loop2];
              elem2 += rlen;
            }

          dest[loop][loop2]=tmp;
        }
    }
#endif
}


/***************** Print_matrix ****************************************/
void Print_matrix(char *str, float **m, int rows)
{
  int   loop, loop2;
  float s=0.0;

  Raport("%s\n", str);
  for(loop=0; loop<=rows; loop++)
    {
      s=0.0;
      for(loop2=0; loop2<=rows; loop2++)
        {
          Raport("%-9.6f ", m[loop][loop2]);
          s+=m[loop][loop2];
        }
      Raport(" S:%-10.6f\n", s);
    }
  Raport("\n");
}

/***************** Print_imatrix ****************************************/
void Print_imatrix(char *str, int **m, int rows, int cols)
{
  int   loop, loop2;
  float s=0.0;

  Raport("%s\n", str);
  for(loop=0; loop<=rows; loop++)
    {
      s=0.0;
      for(loop2=0; loop2<=cols; loop2++)
        {
          Raport("%-4i", m[loop][loop2]);
          s+=m[loop][loop2];
        }
      Raport(" S:%0.0f\n", s);
    }
  Raport("\n");
}

/******************** Get_par *****************************************/
int Get_par(char *txt, int term, char *fmt, ...)
{
  int  loop, loop2, found, row;
  va_list  argptr;
  int  cnt;
  char *e_pos, tmp_str[255];

  found = False;
  strlwr(txt);
  for(loop=0; loop<set_rc; loop++)
    if (!strncmp(set_dat[loop], txt, strlen(txt)))
      {
        row  = loop;
        e_pos=strstr(set_dat[row],"=");
        if (e_pos)
          {
            e_pos++;
            while((*e_pos)==' ')
             e_pos++;
            found= True;
            break;
          }
      }

  if (found)
    {
      va_start(argptr, fmt);
      found = vsscanf(e_pos, fmt, argptr);
      va_end(argptr);
      if (found==EOF)
        found=0;
    }

  if (!found)
    {
      if (term)
        {
          clrscr();
          Raport("Could not find compulsory parameter \"%s\" in the settings file,\nterminating.\n", txt);
          Terminate(1,3,"Invalid settings file.");
        }
      else
        {
          Raport("Could not find parameter \"%s\" in the settings file.\n", txt);
          Raport("=> Using default value %s = ", txt);
        }
    }
  else
    {
//      Raport("%s = %s", txt, e_pos);
    }

  return found;
}

void Fid(int *par, int val)
{
  *par = val;
  Raport("%i \n", val);
}

void Ffd(float *par, float val)
{
  *par = val;
  Raport("%0.4f \n", val);
}

int Get_cpar(char *txt, int term, char def_val, char *par)
{
  int cnt;

  cnt = Get_par(txt, term, "%c", par);

  if (cnt==0)
    {
      *par = def_val;
    }

  *par = toupper(*par);
  if (cnt==0)
    {
      if ((*par)==Yes)
        Raport("Yes \n");
      else if ((*par)==No)
        Raport("No \n");
      else
        Raport("%c \n", *par);
    }
  return cnt;
}


/******************** Read_settings ***********************************/
int Read_settings(int ac)
{
  FILE  *settings_file;
  char  in_line[255], ch;
  int   count, s[21];

  clrscr();
  settings_file=Open_file(sf_name, "r+t", "settings", Terminate_on_error);

  set_dat = cmatrix(0,100,0,255);
  set_rc=0;

  while(fgets(in_line, 255, settings_file))
    {
      strncpy(set_dat[set_rc], in_line, 254);
      strlwr(set_dat[set_rc]);
//      printf("%s", set_dat[set_rc]);
      set_rc++;
    }
  fclose(settings_file);

  if (ac<3)
    Get_par("Output file", True, "%s\n", output_file_name);

  remove(output_file_name);

  sprintf(in_line, "copy %s %s", sf_name, output_file_name);
  system(in_line);
  Raport("\n********************************************************************\n");
  Raport("*  Virtual migration model 95%% confidence limits.                  *\n");
  Raport("*  Ilkka Hanski, Juha Alho and Atte Moilanen.                      *\n");
  Raport("*  Coded by Atte Moilanen 28.04.1997 ->.                           *\n");
  Raport("*  Dept. Ecology & Systematics, Univ. Helsinki, Finland            *\n");
  Raport("********************************************************************\n\n");


//  if (!Get_par("Time steps", False, "%i", &timesteps))
//    Fid(&timesteps, 100);

  Get_par("Data file",   True, "%s\n", data_file_name);
  Get_par("Patch file",  True, "%s\n", patch_file_name);
  Get_par("Marking day file", True, "%s\n", days_file_name);

  Get_par("Function evaluations", True, "%ld\n", &f_evals);
  Get_cpar("B equals -Z", False, No, &z_eq_b);
  Get_cpar("Use edgeness", False, No, &use_edge);
  Get_par("Cap. prob. multip.", True, "%f\n", &cap_pm);
  if (cap_pm!=1.0)
    est_pat1 = True;
  else
    est_pat1 = False;


  Get_par("Mu",   True, "%f\n", &orig_mu);
  Get_par("Eta",  True, "%f\n", &orig_eta);
  Get_par("Zeta", True, "%f\n", &orig_zeta);
  Get_par("Delta",True, "%f\n", &orig_delta);
  Get_par("Beta", True, "%f\n", &orig_beta);
  Get_par("B  ",  True, "%f\n", &orig_B);

  Get_par("Conf. limits for par. no", True, "%i\n", &clfp);
  if ((clfp<0) || (clfp>6))
    {
      Raport("%s\n","Conf. limits can be calculated only to pars. 1-6");
      return False;
    }

  if (z_eq_b==Yes)
    {
      Raport("ZEQB\n");
      if (clfp==6)
        clfp=3;
    }
  else
    Raport("!ZEQB\n");
  if (use_edge==Yes)
    Raport("UE\n");
  else
    Raport("!UE\n");

#if ALPHAVER2
  Raport("Using dispersal kernel 1/pow(1+D,alpha)\n");
#else
  Raport("Using dispersal kernel exp(-alpha*D)\n");
#endif

  div_or_bin = 'R';
  if (div_or_bin=='R')
    Raport("Using random model for daily capture probabilities\n");

  mu   = orig_mu;
  eta  = orig_eta;
  zeta = orig_zeta;
  delta= orig_delta;
  beta = orig_beta;
  B    = orig_B;

  Raport(":%-7.3f eta:%-7.3f zeta:%-7.3f :%-7.3f :%-7.3f B:%-7.3f\n",
          mu, eta, zeta, beta, delta, B);

  free_cmatrix(set_dat, 0, 100, 0, 255);

  return True;
}


/******************** Read_patch_data *****************************************/
int Read_patch_data()
{
  FILE  *data_file;
  char  in_line[255];
  int   line=0, temp, loop;

  min_area=1000000.0;
  data_file=Open_file(patch_file_name, "r+t", "input data", Terminate_on_error);
  Raport("READING PATCH DATA: \n");

  while(fgets(in_line, 255, data_file))
    {
      if (sscanf(in_line, "%f %f %f\n",
                 &patches[line].x_pos, &patches[line].y_pos,
                 &patches[line].area) == 3)
        {

          if (patches[line].area>0.0)
            {
              patches[line].area /= 10000.0;
              if (patches[line].area<min_area)
                min_area = patches[line].area;

              patches[line].pat_num = line+1;
//              Raport("%f %f %f \n", patches[line].x_pos,
//                      patches[line].y_pos, patches[line].area);
              line++;
            }
        }
      else
        {
          fclose(data_file);
          return False;
        }
    }

  pc = line;
  Raport("%i patches read from file\n", pc);
  fclose(data_file);
  return True;
}

/******************** Get_edge_props ************************************/
void Get_edge_props()
{
  int   loop, loop2, pos, loop3;
  float rat, x, y, angle, angles[200], tmp, max_ang;

//  Raport("Edge proportions\n");
  for(loop=0; loop<pc; loop++)
    {
      pos=1;
      for(loop2=0; loop2<pc; loop2++)
        if (loop!=loop2)
          {
            x = patches[loop2].x_pos-patches[loop].x_pos;
            y = patches[loop2].y_pos-patches[loop].y_pos;
            if (x==0)
              {
                if (y<0)
                  angles[pos++] = -M_PI_2;
                else
                  angles[pos++] = M_PI_2;
              }
            else
              angles[pos++]=atan2(y,x);
          }
      for(loop2=1; loop2<pos; loop2++)
        for(loop3=loop2+1; loop3<pos; loop3++)
          if (angles[loop2]>angles[loop3])
            {
              tmp = angles[loop3];
              angles[loop3]  = angles[loop2];
              angles[loop2] = tmp;
            }
      angles[0] = angles[pos-1]-2*M_PI;
      max_ang = 0;
      for(loop2=0; loop2<pos-1; loop2++)
        {
          max_ang = max(max_ang, angles[loop2+1]-angles[loop2]);
//          Raport("E: %-3i %0.4f\n", loop2, angles[loop2]);
        }
      patches[loop].edge_prop = max_ang/(2*M_PI);
//      Raport("%-3i %0.4f\n", loop+1, patches[loop].edge_prop);
    }
}

/******************** Read_day_data *************************************/
int Read_day_data()
{
  FILE  *data_file;
  char  in_line[255];
  int   loop, loop2, temp, day_num, places[500], place_count, place;
  long  row=0;

  data_file=Open_file(days_file_name, "r+t", "input data", Terminate_on_error);
  Raport("\nREADING CAPTURE PLACE DATA: \n");

  day_num = place_count=0;
  for(loop=0; loop<=pc; loop++)
    for(loop2=0; loop2<100; loop2++)
      marked[loop][loop2]=0;

  while(fgets(in_line, 255, data_file))
    {
      row++;
      if ((strlen(in_line)>0) && (in_line[0]!=' '))
        {
          if (in_line[0]=='D')
            {
              if (place_count>0)
                days[day_num].mpv = calloc(place_count+1, sizeof(int));
              else
                days[day_num].mpv = NULL;

              for(loop=0; loop<place_count; loop++)
                {
                  days[day_num].mpv[loop]=places[loop];
                  marked[places[loop]][day_num]++;
                }

              days[day_num].mpc=place_count;
              Raport("Day %i, mark-recapture performed in %i patches\n", day_num, days[day_num].mpc);

              sscanf(in_line, "D %i\n", &day_num);
              place_count = 0;
            }
          else
            {
              sscanf(in_line, "%i\n", &place);
              if (place>pc)
                {
                  Raport("No such patch (%i), row %ld in days file\n",
                          place, row);
                }
              places[place_count++]=place-1;
            }
        }
    }

  if (place_count>0)
    days[day_num].mpv = calloc(place_count+1, sizeof(int));
  else
    days[day_num].mpv = NULL;

  for(loop=0; loop<place_count; loop++)
    {
      days[day_num].mpv[loop]=places[loop];
      marked[places[loop]][day_num]++;
    }

  days[day_num].mpc=place_count;
  Raport("Day %i, mark-recapture performed in %i patches\n", day_num, days[day_num].mpc);

  dayc = day_num;

  fclose(data_file);

  for(loop=1; loop<=dayc; loop++)
    if (days[loop].mpc>0)
      mdc++;

  loop=1;
  do {
      temp=loop;
      do {
        temp++;
      } while ((days[temp].mpc==0) && (temp<dayc));
      days[temp].prev_md = loop;
      days[loop].next_md = temp;
      loop = temp;
  } while (loop<dayc);

  max_surv_int = 0;
  for(loop=1; loop<dayc; loop++)
    if (days[loop].next_md >0)
      max_surv_int = max( days[loop].next_md-loop, max_surv_int);

#if 0
  for(loop=1; loop<dayc; loop++)
    Raport("NPMD: %2i %2i %2i \n", loop, days[loop].prev_md, days[loop].next_md);
  Raport("MSI: %i \n\n", max_surv_int);

  Raport("\n\nMarking day matrix\n");
  for(loop=0; loop<=pc; loop++)
    {
      for(loop2=0; loop2<=dayc; loop2++)
        Raport("%-3i", marked[loop][loop2]);
      Raport("\n");
    }
#endif


  return True;
}


/******************** Read_markings_data *****************************************/
int Read_markings_data()
{
  FILE  *data_file;
  char  in_line[255];
  int   loop, times[500], places[500],
        place_count, bf_num, day_num, place_num, ok;
  long  row=0;

  data_file=Open_file(data_file_name, "r+t", "input data", Terminate_on_error);
  Raport("\nREADING MIGRATION DATA: \n");
  place_count = bfc = 0;
  while(fgets(in_line, 255, data_file))
    {
//      Raport("%p %p %p\n", bflies[0].cpv, bflies[1].ctv, bflies[2].cpv);
      row++;
      if (strlen(in_line)>0)
        {
          if (in_line[0]=='B')
            {
              if (place_count>0)
                {
                  bflies[bfc].cpv = calloc(place_count+1, sizeof(int));
                  bflies[bfc].ctv = calloc(place_count+1, sizeof(int));

                  for(loop=0; loop<place_count; loop++)
                    {
                      bflies[bfc].cpv[loop] = places[loop];
                      bflies[bfc].ctv[loop] = times[loop];
                    }

                  bflies[bfc].cpc=place_count;
                  bflies[bfc].num=bf_num;
//                  Raport("Butterfly %i, recaptured %i times\n",
//                         bf_num, place_count-1);
                  bfc++;
                }
              else;
                bflies[bfc].cpv= bflies[bfc].ctv = NULL;

              sscanf(in_line, "Butterfly %i\n", &bf_num);
              place_count = 0;
            }
          else
            {
              sscanf(in_line, "%i %i\n", &day_num, &place_num);
              if ((place_num>pc) || (place_num<0))
                {
                  Raport("No such patch (%i), row %ld in data file\n",
                          place_num, row);
                }
              if ((day_num<1) || (day_num>dayc))
                {
                  Raport("Invalid day number (%i), row %ld in data file\n",
                          day_num, row);
                }
              ok = False;
              for(loop=0; loop<days[day_num].mpc; loop++)
                if (days[day_num].mpv[loop]==(place_num-1))
                  {
                    ok=True;
                    break;
                  }
              if (!ok)
                {
                  Raport("No capturing done at patch %i, day %i, row %ld in data file\n",
                          place_num, day_num, row);
                }
              else
                {
                  places[place_count]=place_num-1;
                  times[place_count]=day_num;
                  place_count++;
                }
            }
        }
    }

  if (place_count>0)
    {
      bflies[bfc].cpv = calloc(place_count+1, sizeof(int));
      bflies[bfc].ctv = calloc(place_count+1, sizeof(int));

      for(loop=0; loop<place_count; loop++)
        {
          bflies[bfc].cpv[loop] = places[loop];
          bflies[bfc].ctv[loop] = times[loop];
        }

      bflies[bfc].cpc=place_count;
      bflies[bfc].num=bf_num;
//      Raport("Butterfly %i, recaptured %i times\n",
//             bf_num, place_count-1);
      bfc++;
    }
  else;
    bflies[bfc].cpv= bflies[bfc].ctv = NULL;

  Raport("Migration data for %i butterflies read\n", bfc);
  fclose(data_file);
  return True;
}



/******************** Calculate_distances ********************************/
int Calculate_distances()
{
  int   loop2, loop3;
  float dij;

  printf("CALCULATING DISTANCES: ");

//  dist_mat = Lower_triangular_fmatrix(0, pc-1);
  dist_mat = matrix(0, pc-1, 0, pc-1);
  for(loop2=0; loop2<pc; loop2++)
    for(loop3=0; loop3<pc; loop3++)
      {
        dij= sqrt(pow(patches[loop2].x_pos-patches[loop3].x_pos,2)+
                  pow(patches[loop2].y_pos-patches[loop3].y_pos,2)  );
        dist_mat[loop2][loop3] = dij;
      }

  printf("Ok. Done.\n\n");
  return True;
}

void Get_isols()
{
  int loop, loop2;

  for(loop=0; loop<pc; loop++)
    {
      patches[loop].isolation = 0.0;
      for(loop2=0; loop2<pc; loop2++)
        if (loop!=loop2)
          {
#if ALPHAVER2
            patches[loop].isolation +=
              1.0/(pow(1.0+dist_mat[loop][loop2],beta))*pow(patches[loop2].area, B);
#else
            patches[loop].isolation +=
              exp(-beta*dist_mat[loop][loop2])*pow(patches[loop2].area, B);
#endif
          }
    }
}

void Calc_pim()
{
  int loop, loop2;

  for(loop=0; loop<pc; loop++)
    for(loop2=0; loop2<pc; loop2++)
      if (loop2!=loop)
        {
#if ALPHAVER2
          pim[loop][loop2] =
              1.0/(pow(1.0+dist_mat[loop][loop2],beta))
                  *pow(patches[loop2].area, B)/patches[loop].isolation;
#else
          pim[loop][loop2] =
            exp(-beta*dist_mat[loop][loop2])*pow(patches[loop2].area,B)/
                                             patches[loop].isolation;
#endif
        }
      else
        pim[loop][loop2] = 0.0;
}


/******************** Calc_P *********************************************/
void Calc_P()
{
  int   loop, loop2, loop3;
  float d;

  Get_isols();
  Calc_pim();
  for(loop=0; loop<pc; loop++)
    {
      d = delta*delta/(delta*delta+pow(patches[loop].isolation,2));
      if (use_edge==Yes)
        patches[loop].mu = 1.0-(1.0-patches[loop].edge_prop)*(1.0-d);
      else
        patches[loop].mu = d;
    }

  P[0][0]=1.0;
  for(loop=1; loop<=pc; loop++)
    P[0][loop]=0.0;

  for(loop=1; loop<=pc; loop++)
    P[loop][0] = mu + (1.0-mu)*eta * pow(patches[loop-1].area, zeta)*
                                   patches[loop-1].mu;

  for(loop=1; loop<=pc; loop++)
    for(loop2=1; loop2<=pc; loop2++)
      if (loop2!=loop)
        P[loop][loop2] = (1.0-mu)*eta* pow(patches[loop-1].area, zeta)*
                         (1.0-patches[loop-1].mu)*pim[loop-1][loop2-1];

//  Raport(":%-7.3f eta:%-7.3f zeta:%-7.3f :%-7.3f :%-7.3f B:%-7.3f \n",
//         mu, eta, zeta, beta, delta, B);

  for(loop=1; loop<=pc; loop++)
    {
      P[loop][loop] = (1.0-mu)*(1.0-eta*pow(patches[loop-1].area, zeta));
//      Raport("%f %f \n", patches[loop-1].area,pow(patches[loop-1].area, zeta));
    }

//  for(loop=0; loop<=pc; loop++)
//    for(loop2=0; loop2<=pc; loop2++)
//      P_vec[1][loop][loop2] = P[loop][loop2];
  Mat_cpy(P, P_vec[1], pc);

  for(loop=2; loop<=max_surv_int; loop++)
    Mat_multip(P_vec[loop], P_vec[loop-1], P, pc);
}



/******************** Get_h *******************************************/
void Get_h()
{
  int   loop, loop2, day;
  float multip, tmp, *fpt, **htmp;

  if (pri)
    {
      Print_matrix("P^1",P_vec[1], pc);
      Raport("\n");
      for(loop2=1; loop2<=dayc; loop2++)
        Raport("%-6.3f ", days[loop2].ro);
      Raport("\n");
      for(loop=0; loop<pc; loop++)
        {
           for(loop2=1; loop2<=dayc; loop2++)
             Raport("%-6.3f ", patches[loop].ro[loop2]);
           Raport("\n");
        }
    }

//  Diag(H2, pc);

  for(day=dayc-1; day>0; day--)
    {
#if 0
      Mat_cpy(P_vec[1], H3, pc);

      for(loop=1; loop<=pc; loop++)
        for(loop2=1; loop2<=pc; loop2++)
          H3[loop][loop2] *= (1.0- patches[loop2-1].ro[day+1]);
#endif
      if (day == (dayc-1))
        Mat_cpy(days[day+1].p1r, H4, pc);
      else
        Mat_multip(H4, days[day+1].p1r, H2, pc);

      for(loop=0; loop<=pc; loop++)
        {
          tmp = 0.0;
          fpt = H4[loop];
          for(loop2=0; loop2<=pc; loop2++)
            tmp += fpt[loop2];
          H[loop][day] = tmp;
        }

      htmp = H2;
      H2 = H4;
      H4 = htmp;
//      Mat_cpy(H4, H2, pc);

      if (pri)
        {
          Print_matrix("H2 ",H2, pc);
          Print_matrix("H3 ",H3, pc);
          Print_matrix("H4 ",H4, pc);
        }
    }

  for(loop=0; loop<=pc; loop++)
    {
      H[loop][dayc]=1.0;
      H[loop][0]=H[loop][1];
    }

  if (pri)
    {
      Raport("\n");
      for(loop=0; loop<=pc; loop++)
        {
           for(loop2=1; loop2<=dayc; loop2++)
             Raport("%-6.3f ", H[loop][loop2]);
           Raport("\n");
        }
      pri=False;
    }
}

void Fix_lhs(int day, int day2)
{
  int    loop, loop2;
  struct butterfly *b;

  for(loop=0; loop<bfc; loop++)
    {
      b = &bflies[loop];
      if (b->cpc>1)
        {
          for(loop2=0; loop2<(b->cpc-1); loop2++)
            if ( (b->ctv[loop2]==day) && (b->ctv[loop2+1]==day2) )
              {
//                printf("Found bf %i %i %i \n", b->num, day, day2);
                b->lh *= H4[1+b->cpv[loop2]][1+b->cpv[loop2+1]];
                b->oper++;
                break;
              }
        }
    }
}

/******************* Get_p1r ***************************************/
void Get_p1r()
{
  int day, loop, loop2;
  float **m;

  for(day=1; day<=dayc; day++)
    {
      m = days[day].p1r;
      Mat_cpy(P_vec[1], m, pc);

      for(loop=1; loop<=pc; loop++)
        for(loop2=1; loop2<=pc; loop2++)
          m[loop][loop2] *= (1.0- patches[loop2-1].ro[day]);
    }
}

/******************** Get_lh *********************************************/
float Get_lh()
{
  int    loop, loop2, day, day2,ok;
  struct butterfly *bf;
  float  **htmp, ftmp;
  double lh;

  ok = True;
  for(loop=0; loop<bfc; loop++)
    {
      bf = &bflies[loop];
      bf->oper=0;
      bf->lh = H[bf->cpv[bf->cpc-1]+1][bf->ctv[bf->cpc-1]];
      if ((bf->lh >1) || (bf->lh<=0))
        {
          printf("b->lh %f (%i %i) out of range\n", bf->lh,
          bf->cpv[bf->cpc-1]+1, bf->ctv[bf->cpc-1]);
          ok = False;
        }

    }

  if (0 && !ok)
    {
      Print_matrix("P^1",P_vec[1], pc);
      Raport("\n\nH\n");
      for(loop=0; loop<=pc; loop++)
        {
           for(loop2=1; loop2<=dayc; loop2++)
             Raport("%-6.3f ", H[loop][loop2]);
           Raport("\n");
        }

      Raport("\nro\n\n");
      for(loop=0; loop<pc; loop++)
        {
           for(loop2=1; loop2<=dayc; loop2++)
             Raport("%-6.3f ", patches[loop].ro[loop2]);
           Raport("\n");
        }
    }

  for(day=1; day<dayc; day++)
    {
      day2=day+1;
//      Diag(H2, pc);
      while(day2<=dayc)
        {
          if (QCM[day][day2]>0)
            {
              Mat_cpy(P_vec[1], H3, pc);
              for(loop=0; loop<=pc; loop++) ///???
                H3[loop][0] = 0;

              for(loop2=1; loop2<=pc; loop2++)
                {
                  ftmp = patches[loop2-1].ro[day2];
                  for(loop=1; loop<=pc; loop++)
                    H3[loop][loop2] *= ftmp;
                }

              if (day2 == (day+1))
                Mat_cpy(H3, H4, pc);
              else
                Mat_multip(H4, H2, H3, pc);

              Fix_lhs(day, day2);

            }
#if 0
          Mat_cpy(P_vec[1], H3, pc);
          for(loop=1; loop<=pc; loop++)
            for(loop2=1; loop2<=pc; loop2++)
              H3[loop][loop2] *= (1.0- patches[loop2-1].ro[day2]);
#endif
          if (QCM[day][day2]==2)
            break;

          if (day2 == (day+1))
            Mat_cpy(days[day2].p1r, H4, pc);
          else
            Mat_multip(H4, H2, days[day2].p1r, pc);

          htmp = H2; H2 = H4; H4 = htmp;
//          Mat_cpy(H4, H2, pc);
          day2++;
        }
    }

  lh = 0.0;
  for(loop=0; loop<bfc; loop++)
    {
      if ((bflies[loop].lh >1.000001) || (bflies[loop].lh<0))
        {
//          Raport("blh %lf out of range\n", bflies[loop].lh);
//          printf("B%0.4f ", bflies[loop].lh);
          bflies[loop].lh = 0;
        }
      if (bflies[loop].lh==0.0)
        lh -= 100;
      else
        lh += log(bflies[loop].lh);
    }

  return (float)lh;
}


/******************** Fix_qcm ********************************************/
void Fix_qcm()
{
  int loop, loop2;

  for(loop=0; loop<=dayc; loop++)
    for(loop2=0; loop2<=dayc; loop2++)
      QCM[loop][loop2] = 0;

  for(loop=0; loop<bfc; loop++)
    if (bflies[loop].cpc>0)
      {
        for(loop2=0; loop2<(bflies[loop].cpc-1); loop2++)
          QCM[bflies[loop].ctv[loop2]][bflies[loop].ctv[loop2+1]]=1;
      }

  for(loop=0; loop<=dayc; loop++)
    for(loop2=dayc; loop2>=0; loop2--)
      if (QCM[loop][loop2]==1)
        {
          QCM[loop][loop2]=2;
          break;
        }
}

void Fix_fixed_vec()
{
  int loop;

  for(loop=0; loop<10; loop++)
    fixed_vec[loop]=0;

  fixed_vec[clfp]=1;

  if (((clfp==3) || (clfp==6)) && (z_eq_b==Yes))
    fixed_vec[3]=fixed_vec[6]=1;

  Raport("\n\nParameters to check:\n");
  for(loop=1; loop<=6; loop++)
    Raport("%i ", fixed_vec[loop]);
  Raport("\n");

}


/********************* Fix_nit_mat **********************************/
void Fix_nit_mat()
{
  int loop, loop2, loop3, p1, p2, t1, t2;

  for(loop=1; loop<=dayc; loop++)
    if (days[loop].mpc>0)
      {
        days[loop].ni1t  = calloc(pc+1, sizeof(int));
        days[loop].ni2t  = calloc(pc+1, sizeof(int));
        days[loop].N     = calloc(pc+1, sizeof(int));
        days[loop].gi1t  = calloc(pc+1, sizeof(float));
        days[loop].gi2t  = calloc(pc+1, sizeof(float));
        days[loop].vi1t  = calloc(pc+1, sizeof(float));
        days[loop].vi2t  = calloc(pc+1, sizeof(float));
        days[loop].nt1   = 0;
        days[loop].nt2   = 0;
      }
    else
      {
        days[loop].ni1t = NULL;
        days[loop].ni2t = NULL;
        days[loop].gi1t = NULL;
        days[loop].gi2t = NULL;
        days[loop].vi1t = NULL;
        days[loop].vi2t = NULL;
      }

  for(loop=0; loop<bfc; loop++)
    if (bflies[loop].cpc>1)
      {
        for(loop2=0; loop2<(bflies[loop].cpc-1); loop2++)
          {
            p1 = bflies[loop].cpv[loop2];
            p2 = bflies[loop].cpv[loop2+1];
            t1 = bflies[loop].ctv[loop2];
            t2 = bflies[loop].ctv[loop2+1];
            if (t1 == days[t2].prev_md)
              {
                if (est_pat1==False)
                  {
                    days[t2].ni1t[p1]++;
                    days[t2].nt1++;
                  }
                else
                  {
                    if (p2>0)
                      {
                        days[t2].ni2t[p1]++;
                        days[t2].nt2++;
                      }
                    else
                      {
                        days[t2].ni1t[p1]++;
                        days[t2].nt1++;
                      }
                  }
              }
          }
      }

  for(loop=0; loop<bfc; loop++)
    if (bflies[loop].cpc>0)
      {
        for(loop2=0; loop2<bflies[loop].cpc; loop2++)
          {
            p1 = bflies[loop].cpv[loop2];
            t1 = bflies[loop].ctv[loop2];
            days[t1].N[p1]++;
          }
      }
#if 0
  Raport("ni1t\n");
  for(loop=0; loop<=dayc; loop++)
    {
      if (days[loop].mpc>0)
        {
          Raport("DAY %2i ", loop);
          for(loop2=0; loop2<=pc; loop2++)
            Raport("%2i ", days[loop].ni1t[loop2]);
          Raport("\n");
        }
    }
  Raport("\n");
  Raport("ni2t\n");
  for(loop=0; loop<=dayc; loop++)
    {
      if (days[loop].mpc>0)
        {
          Raport("DAY %2i ", loop);
          for(loop2=0; loop2<=pc; loop2++)
            Raport("%2i ", days[loop].ni2t[loop2]);
          Raport("\n");
        }
    }
  Raport("\n");
  Raport("\n");
  Raport("N\n");
  for(loop=0; loop<=dayc; loop++)
    {
      if (days[loop].mpc>0)
        {
          Raport("DAY %2i ", loop);
          for(loop2=0; loop2<=pc; loop2++)
            Raport("%2i ", days[loop].N[loop2]);
          Raport("\n");
        }
    }
  Raport("\n");
#endif

}

/********************* Fix_git ***************************************/
void Get_git()
{
  int loop, loop2, loop3, p1, i, time_int;

  for(loop=0; loop<=dayc; loop++)
    if (days[loop].mpc)
      {
        time_int = loop - days[loop].prev_md;
        for(loop3=0; loop3<=pc; loop3++)
          {
            days[loop].gi1t[loop3] = 0.0;
            days[loop].gi2t[loop3] = 0.0;
          }

        for(loop2=0; loop2<days[loop].mpc; loop2++)
          {
            p1 = days[loop].mpv[loop2]+1;
//            printf("%i ", p1);
            if (est_pat1==False)
              {
                for(loop3=0; loop3<=pc; loop3++)
                  days[loop].gi1t[loop3] += P_vec[time_int][loop3][p1];
              }
            else
              {
                if (p1>1)
                  {
                    for(loop3=0; loop3<=pc; loop3++)
                      days[loop].gi2t[loop3] += P_vec[time_int][loop3][p1];
                  }
                else
                  {
//                    printf("H");
                    for(loop3=0; loop3<=pc; loop3++)
                      {
                        days[loop].gi1t[loop3] += P_vec[time_int][loop3][p1];
//                        printf("%0.5f ",P_vec[time_int][loop3][p1]);
                      }
                  }
              }
          }
        for(loop3=0; loop3<=pc; loop3++)
          {
            if (days[loop].gi1t[loop3] > 1.0 )
              printf("GE");
//            else if (days[loop].gi1t[loop3] == 0.0 )
//              printf("Z(%i) ", loop3);
          }

      }
}



/********************* Fix_vit ***************************************/
void Get_vit()
{
  int loop, loop2, loop3, p1, i, time_int;

  for(loop=0; loop<=dayc; loop++)
    if (days[loop].mpc)
      {
        time_int = loop - days[loop].prev_md;
        for(loop3=0; loop3<=pc; loop3++)
          days[loop].vi1t[loop3] = 0.0;

        for(loop2=0; loop2<days[loop].mpc; loop2++)
          {
            p1 = days[loop].mpv[loop2]+1;
//            printf("%i ", p1);
            if (est_pat1==False)
              {
                for(loop3=0; loop3<=pc; loop3++)
                  days[loop].vi1t[loop3] += P_vec[time_int][loop3][p1]*obs_mean1;
              }
            else
              {
                if (p1>1)
                  {
                    for(loop3=0; loop3<=pc; loop3++)
                      days[loop].vi1t[loop3] += P_vec[time_int][loop3][p1]*obs_mean2;
                  }
                else
                  {
                    for(loop3=0; loop3<=pc; loop3++)
                      days[loop].vi1t[loop3] += P_vec[time_int][loop3][p1]*obs_mean1;
                  }
              }
          }
        for(loop3=0; loop3<=pc; loop3++)
          if (days[loop].vi1t[loop3] > 1.0 )
            printf("V ");
      }
}

/************************** Get_obs_mean ***************************/
void Get_obs_mean()
{
  int tot_obs1, tot_obs2, loop, loop2, loop3, t1, t2, *mv1;
  struct m_day *d1, *d2;
  float divisor1=0.0, divisor2=0.0;

  tot_obs1 = tot_obs2 = 0;
  for(loop=2; loop<=dayc; loop++)
    {
      tot_obs1 += days[loop].nt1;
      tot_obs2 += days[loop].nt2;
    }

//  printf("tot %i %i\n", tot_obs1, tot_obs2);

  for(loop=2; loop<=dayc; loop++)
    {
      if ((t1=days[loop].prev_md)>0)
        {
          t2 = loop;
          d1= &days[t1];
          d2= &days[t2];
          mv1 = d1->mpv;
          if (est_pat1==False)
            {
              for(loop2=0; loop2<d1->mpc; loop2++)
                divisor1 += d1->N[mv1[loop2]]*d2->gi1t[mv1[loop2]+1];
            }
          else
            {
              for(loop2=0; loop2<d1->mpc; loop2++)
                {
                  if (mv1[loop2]==0)
                    divisor1 += d1->N[mv1[loop2]]*d2->gi1t[mv1[loop2]+1];
                  else
                    divisor2 += d1->N[mv1[loop2]]*d2->gi2t[mv1[loop2]+1];
                }
            }
        }
    }

//printf("DIV:%f %f \n\n", divisor1, divisor2);
  obs_mean1 = tot_obs1 / divisor1;
  if (divisor2>0.0)
    obs_mean2 = tot_obs2 / divisor2;
  else
    obs_mean2 = 0.0;
//  printf("OM: %0.4f %0.4f\n", obs_mean1, obs_mean2);
}

/****************** Get_E_et_Vs **************************************/
void Get_E_et_Vs()
{
  int loop, loop2, loop3, p1, t1;
  float tmp, nume=0.0, divi=0.0;

  for(loop=2; loop<=dayc; loop++)
    if (t1=days[loop].prev_md)
      {
        days[loop].V1 =days[loop].V2 =days[loop].V3 =days[loop].ENt =0.0;

        for(loop2=0; loop2<days[t1].mpc; loop2++)
          {
            p1 = days[t1].mpv[loop2]+1;
            days[loop].ENt+= days[t1].N[p1-1]*days[loop].vi1t[p1];
            days[loop].V1 += days[t1].N[p1-1]
                           *(days[loop].gi1t[p1]+days[loop].gi2t[p1]);
//printf("%i  %0.3f  %0.3f %0.3f\n",days[t1].N[p1-1],
//                           days[loop].gi1t[p1],days[loop].gi2t[p1], days[loop].V1);
            days[loop].V3 += days[t1].N[p1-1]
                           *pow(days[loop].gi1t[p1]+days[loop].gi2t[p1],2.0);
            days[loop].V2 += days[t1].N[p1-1]
                           *(days[loop].vi1t[p1]-pow(days[loop].vi1t[p1],2.0));
          }
//nume += pow(days[loop].nt1+days[loop].nt2,2.0)-pow(days[loop].ENt,2.0)-days[loop].V2;
//divi += days[loop].V1*days[loop].V1-days[loop].V3;
//tmp = (pow(days[loop].nt1+days[loop].nt2,2.0)-pow(days[loop].ENt,2.0)-days[loop].V2)
//      /(days[loop].V1*days[loop].V1-days[loop].V3);
//printf("d:%i  v%0.3f v%0.3f v%0.3f e%0.3f n%i = %0.3f  %0.3f  %0.3f\n",
//  loop, days[loop].V1,days[loop].V2,
//  days[loop].V3,days[loop].ENt,days[loop].nt1+days[loop].nt2, tmp,nume,divi);
      }
//printf("xx%0.3f\n", nume, divi);
  if (0 && ntprns==True)
    {
      Raport("nt1  nt2  ENt  V1  V2  V3\n");
      for(loop=2; loop<=dayc; loop++)
        {
          Raport("%2i %4i  %4i  %8.4f  %8.4f  %8.4f  %8.4f\n", loop,
                 days[loop].nt1, days[loop].nt2,
                 days[loop].ENt, days[loop].V1,
                 days[loop].V2, days[loop].V3);
        }
    }
}


/********************* Get_obs_std ***********************************/
void Get_obs_std()
{
  int loop, loop2, loop3, p1, t1;
  float numerator=0.0, divisor=0.0;
  struct m_day *d;

  for(loop=2; loop<=dayc; loop++)
    if (days[loop].mpc)
      {
        d = &days[loop];

        numerator += pow(d->nt1 + d->nt2 - d->ENt,2.0) - d->V2;
        divisor   += pow(d->V1 ,2.0) - d->V3;
      }

  obs_std = numerator / divisor;

//  printf("std: %0.4f ", obs_std);
}


/********************* Get_var_et_cov ********************************/
void Get_VAR_et_COV()
{
  int loop, loop2;
  struct m_day *d;

  for(loop=2; loop<=dayc; loop++)
    {
      d = &days[loop];

      d->EVARt = (pow(d->V1,2.0) - d->V3)*obs_std+d->V2;
      d->ECOVt = d->V1*obs_std;

      if (d->EVARt==0.0)
        {
          d->ksi = 0.0;
//          printf("Z");
        }
      else
        d->ksi   = d->ECOVt/d->EVARt*(d->nt1+d->nt2-d->ENt);

      if (est_pat1==False)
        {
          d->ro = obs_mean1 + d->ksi;
          if (d->ro <0.0)
            d->ro = 0.0;
          else if (d->ro >0.9999)
            d->ro = 0.99999;
          d->ro1=0.0;
        }
      else
        {
          d->ro1 = obs_mean1 + d->ksi;
          if (d->ro1 <0.0)
            d->ro1 = 0.0;
          else if (d->ro1 >0.9999)
            d->ro1 = 0.99999;

          d->ro = obs_mean2 + d->ksi;
          if (d->ro <0.0)
            d->ro = 0.0;
          else if (d->ro >0.9999)
            d->ro = 0.99999;
        }
//      printf("%2i %0.3f %0.3f\n", loop, d->ro, d->ro1);
//      d->ro =
    }

  for(loop=0; loop<pc; loop++)
    {
       patches[loop].ro[1] = 0.0;
       for(loop2=2; loop2<=dayc; loop2++)
         {
           if (marked[loop][loop2])
             {
               if (est_pat1==False)
                 patches[loop].ro[loop2] = days[loop2].ro;
               else
                 {
                   if (loop==0)
                     patches[loop].ro[loop2] = days[loop2].ro1;
                   else
                     patches[loop].ro[loop2] = days[loop2].ro;
                 }
             }
           else
             {
               patches[loop].ro[loop2] = 0.0;
             }
         }
    }
}


/********************* Ecp_random_model ******************************/
void Ecp_random_model()
{
   Get_git();

   Get_obs_mean();

   Get_vit();

   Get_E_et_Vs();

   Get_obs_std();

   Get_VAR_et_COV();
}


/******************** Free_data ******************************************/
void Free_data()
{
  int loop;

  Rap();
//  Free_ltfm(dist_mat, 0, pc-1);
  free_matrix(dist_mat, 0, pc-1, 0, pc-1);

  for(loop=1; loop<=dayc; loop++)
    {
      free(days[loop].mpv);
      free(days[loop].ni1t);
      free(days[loop].ni2t);
      free(days[loop].N);
      free(days[loop].gi1t);
      free(days[loop].gi2t);
      free(days[loop].gi1t);
      free(days[loop].gi2t);
    }
  free(days);

  for(loop=0; loop<pc; loop++)
    {
      free(patches[loop].mdv);
      free(patches[loop].ro);
    }
  free(patches);

  for(loop=0; loop<bfc; loop++)
    {
      free(bflies[loop].cpv);
      free(bflies[loop].ctv);
    }
  free(bflies);
  free_matrix(P,   0, pc, 0, pc);
  free_matrix(Q,   0, pc, 0, pc);
  free_imatrix(QCM,0, dayc,0,dayc);
  free_matrix(H2,  0, pc, 0, pc);
  free_matrix(H3,  0, pc, 0, pc);
  free_matrix(H4,  0, pc, 0, pc);
  free_matrix(pim, 0, pc, 0, pc);
  free_matrix(H,   0, pc, 0, dayc);
  free_imatrix(obs_mat, 0, pc, 0, dayc);
  free_imatrix(live_mat,0, pc, 0, dayc);
  free_imatrix(marked,0, pc, 0, 100);

  for(loop=1; loop<=dayc; loop++)
    {
      free_matrix(P_vec[loop], 0, pc, 0, pc);
      free_matrix(days[loop].p1r, 0, pc, 0, pc);
    }

  Rap();
}


/****************************** CL95_est ******************************/
void CL95_est(float *par, float orig_val, float low_lim, float upp_lim,
              float acc,  char *name)
{
  struct est_point
    {
      float par_val;
      float est;
      int   repl;
      int   evals;
    };

  const float TARG = 1.9205;

  struct est_point points[100];

  float cur, prev, closest, closest_err, err,
        min_val, max_val, min_change, change;
  int   iters;


  if (fabs(orig_val)>10.0)
    min_change = 0.1;
  else if (fabs(orig_val)>1.0)
    min_change = 0.01;
  else if (fabs(orig_val)>0.1)
    min_change = 0.001;
  else
    min_change = 0.001;


  Raport("Computing confidence limits for parameter %s\n", name);
  Raport("Difference between model fit error at optimum parameters\n");
  Raport("and at the confidence limit is 1.9205 (3.841/2).\n");
  Raport("The confidence limits are located numerically.\n\n");
  if (orig_val==low_lim)
    {
      Raport("*** %s lower 95%% limit = %0.5f\n\n", name, low_lim);
      goto UPPER;
    }
#if 0
  Raport("Searching for upper limit for parameter %s lower confidence \n", name);
  Raport("interval without reoptimization of free parameters (other than %s).\n\n",name);
  cur = orig_val;
  do {
    prev = cur;

    change = max(0.05*cur, min_change);
    cur -= change;

    if (cur<low_lim)
     cur = low_lim;
    *par = cur;

    err = Get_error_old();
    Raport("Lower: %9.4f %8.4f %8.4f\n", cur, err, (err-orig_fit)/2.0 );

    if (cur<=low_lim)
      {
        if ((err-orig_fit)<TARG)
          {
            Raport("*** %s lower 95%% limit = %0.5f\n\n", name, low_lim);
            goto UPPER;
          }
        break;
      }

  } while ((err-orig_fit)<TARG);

  iters = 0;
  *par = max_val = prev;
  Raport("Upper limit for lower CL for %s is %0.5f\n\n", name,prev);
#endif
  Raport("Searching for lower confidence limit with \n");
  Raport("parameters reoptimized using %i function evaluations.\n\n", f_evals);


  cur = Estimate2(f_evals, prev, &err, orig_fit+TARG, 1);
  Raport("\n\n*** %s lower 95%% limit = %0.5f \n\n",
         name, cur);


UPPER:
  Raport("Searching for upper confidence limit with \n");
  Raport("parameters reoptimized using %i function evaluations.\n\n", f_evals);
  cur = Estimate2(f_evals, prev, &err, orig_fit+TARG, 0);
  Raport("\n\n*** %s upper 95%% limit = %0.5f \n\n",
         name, cur);

}


void Get_cls_new()
{
  orig_fit = Get_error_old();
  Raport("Error at optimum = %0.5f\n", orig_fit);
  switch (clfp)
    {
      case 1 :
        {
          CL95_est(&mu, orig_mu, 0, 1, 0.05, "mu");
          break;
        }
      case 2 :
        {
          CL95_est(&eta, orig_eta, 0, 1, 0.05, "eta");
          break;
        }
      case 3 :
        {
          CL95_est(&zeta, orig_zeta, -2, 0, 0.05, "zeta");
          break;
        }
      case 4 :
        {
          CL95_est(&beta, orig_beta, 0, 100, 0.05, "beta");
          break;
        }
      case 5 :
        {
          CL95_est(&delta, orig_delta, 0, 100, 0.05, "delta");
          break;
        }
      case 6 :
        {
          CL95_est(&B, orig_B, 0, 2, 0.05, "B");
          break;
        }
    }
}


/********************* main *******************************************/
int main(int argc, char **argv)
{
  int   loop, loop2;
  float ave, var;

  ntprns=False;
  strcpy(sf_name, "settings.dat");
  strcpy(output_file_name, "results");
  if (argc>=2)
    strncpy(sf_name,  argv[1], 13);
  if (argc>=3)
    strncpy(output_file_name, argv[2], 13);

  randomize();
  idnum=-101-random(32600);
//  idnum=-7;
  ran1(&idnum);

  if (!Read_settings(argc))
    Terminate(1,2, "Invalid settings file -settings.dat");


  if ( !(patches=calloc(300, sizeof(struct patch))) )
    Terminate(1,2, "Error allocating patches");

  if ( !(days=calloc(365, sizeof(struct m_day))) )
    Terminate(1,2, "Error allocating days");

  if ( !(bflies=calloc(2500, sizeof(struct butterfly))) )
    Terminate(1,2, "Error allocating butterflies");

  for(loop=0; loop<2500; loop++)
    bflies[loop].cpc=0;

  if (!Read_patch_data())
    Terminate(1,2, "Invalid patch data file ");

  if (!(marked = imatrix(0, pc, 0, 100)))
    Terminate(1,2, "Error allocating day mat ");

  if (!Read_day_data())
    Terminate(1,2, "Invalid day data file ");

  matr_size = ((pc+1)*((long)pc+1)+1)*sizeof(float);
  if (!(P = matrix(0, pc, 0, pc)))
    Terminate(1,2, "Error allocating P ");

  if (!(Q = matrix(0, pc, 0, pc)))
    Terminate(1,2, "Error allocating Q ");

  if (!(QCM = imatrix(0, dayc, 0, dayc)))
    Terminate(1,2, "Error allocating QCM ");

  if (!(H2= matrix(0, pc, 0, pc)))
    Terminate(1,2, "Error allocating H2 ");

  if (!(H3= matrix(0, pc, 0, pc)))
    Terminate(1,2, "Error allocating H3 ");

  if (!(H4= matrix(0, pc, 0, pc)))
    Terminate(1,2, "Error allocating H4 ");

  if (!(H = matrix(0, pc, 0, dayc)))
    Terminate(1,2, "Error allocating H ");

  if (!(obs_mat = imatrix(0, pc, 0, dayc)))
    Terminate(1,2, "Error allocating H ");

  if (!(live_mat = imatrix(0, pc, 0, dayc)))
    Terminate(1,2, "Error allocating H ");

  if (!(pim = matrix(0, pc, 0, pc)))
    Terminate(1,2, "Error allocating pim ");


  for(loop=1; loop<=dayc; loop++)
    {
    if (!(P_vec[loop] = matrix(0, pc, 0, pc)))
      {
        Raport("Error allocating p-matrix %i\n", loop);
        Terminate(1,2, "Error allocating P ");
      }

    if (!(days[loop].p1r = matrix(0, pc, 0, pc)))
      {
        Raport("Error allocating p1r-matrix %i\n", loop);
        Terminate(1,2, "Error allocating p1r ");
      }
    }

  for(loop=0; loop<pc; loop++)
    {
      patches[loop].ro       = calloc(dayc+1, sizeof(float));
    }

  if (!Calculate_distances())
    Terminate(1,2, "Error calculating distances ");

  Get_edge_props();

  if (!Read_markings_data())
    Terminate(1,2, "Invalid markings data file ");

  Fix_nit_mat();

  Fix_qcm();

//  Print_imatrix("QCM",QCM, dayc, dayc);

  Fix_fixed_vec();

  Get_cls_new();

  Free_data();

  if (!rap_mode)
    Close_rap_file();

  fcloseall();

  return EXIT_SUCCESS;
}

