/**************************************************************************/
/*                                                                        */
/*              Public domain memory allocation utilities from            */
/*              Press et. al Numerical Algorithms in C 2nd ed.            */
/*                                                                        */
/**************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <alloc.h>
#define NR_END 1
#define FREE_ARG char*

void   nrerror(char error_text[]) {
fprintf(stderr, "Numerical receipes run-time error ...\n");
fprintf(stderr, "%s\n", error_text);
fprintf(stderr, "... now exiting to system ...\n");
exit(1);
}



void Mat_multip(float **dest, float **m1, float **m2, int rows)
{
  int loop, loop2, loop3;

  for(loop=0; loop<=rows; loop++)
    for(loop2=0; loop2<=rows; loop2++)
      {
        dest[loop][loop2] = 0.0;
        for(loop3=0; loop3<=rows; loop3++)
          dest[loop][loop2] += m1[loop][loop3]*m2[loop3][loop2];
      }
}


float  *vector(long nl, long nh) {
float *v;

v=(float *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(float)));
if (!v) nrerror("Allocation error in vector()");
return v-nl+NR_END;
}


int    *ivector(int nl, int nh) {
int    *v;

v=(int *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(int)));
if (!v) nrerror("Allocation error in ivector()");
return v-nl+NR_END;
}


float  **matrix( long nrl, long nrh, long ncl, long nch) {
long i, nrow=nrh-nrl+1, ncol=nch-ncl+1;
float **m;

m=(float **) malloc((size_t)((nrow+NR_END)*sizeof(float*)));
if (!m) nrerror("Allocation error 1 in matrix()");
m += NR_END;
m -= nrl;

m[nrl]=(float *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(float)));
if (!m[nrl]) nrerror("Allocation error 2 in matrix()");
m[nrl] += NR_END;
m[nrl] -= ncl;

for(i=nrl+1; i<=nrh; i++) m[i]=m[i-1]+ncol;
return m;
}


char  **cmatrix( long nrl, long nrh, long ncl, long nch) {
long i, nrow=nrh-nrl+1, ncol=nch-ncl+1;
char **m;

m=(char **) malloc((size_t)((nrow+NR_END)*sizeof(char *)));
if (!m) nrerror("Allocation error 1 in cmatrix()");
m += NR_END;
m -= nrl;

m[nrl]=(char *) calloc(1, (size_t)((nrow*ncol+NR_END)*sizeof(char)));
if (!m[nrl]) nrerror("Allocation error 2 in cmatrix()");
m[nrl] += NR_END;
m[nrl] -= ncl;

for(i=nrl+1; i<=nrh; i++) m[i]=m[i-1]+ncol;
return m;
}

int    **imatrix(long nrl, long nrh, long ncl, long nch) {
long i, nrow=nrh-nrl+1, ncol=nch-ncl+1;
int **m;

m=(int **) malloc((size_t)((nrow+NR_END)*sizeof(int *)));
if (!m) nrerror("Allocation error 1 in imatrix()");
m += NR_END;
m -= nrl;

m[nrl]=(int *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(int)));
if (!m[nrl]) nrerror("Allocation error 2 in imatrix()");
m[nrl] += NR_END;
m[nrl] -= ncl;

for(i=nrl+1; i<=nrh; i++) m[i]=m[i-1]+ncol;
return m;
}

#if 0

float  **submatrix(float **a, long oldrl, long oldrh, long oldcl, long oldch,
                   long newrl, long newcl) {
long i,j, nrow=oldrh-oldrl+1, ncol=oldcl-newcl;
float **m;

m=(float **) malloc((size_t) ((nrow+NR_END)*sizeof(float*)));
if (!m) nrerror("Allocation error in submatrix()");
m += NR_END;
m -= newrl;

for(i=oldrl, j=newrl; j<=oldrh;  i++, j++) m[j]=a[i]+ncol;
return m;
}

float  **convert_matrix(float *a, long nrl, long nrh, long ncl, long nch) {
long i,j, nrow=nrh-nrl+1, ncol=nch-ncl+1;
float **m;

m=(float **) malloc((size_t) ((nrow+NR_END)*sizeof(float*)));
if (!m) nrerror("Allocation error in convert_matrix()");
m += NR_END;
m -= nrl;

m[nrl]=a-ncl;

for(i=1, j=nrl+1; i<=nrow;  i++, j++) m[j]=m[j-1]+ncol;
return m;
}


float  ***f3tensor(long nrl, long nrh, long ncl, long nch, long ndl, long ndh) {
long i,j, nrow=nrh-nrl+1, ncol=nch-ncl+1, ndep=ndh-ndl+1;
float ***t;

t =(float ***) malloc((size_t) ((nrow+NR_END)*sizeof(float**)));
if (!t) nrerror("Allocation error 1 in f3tensor()");
t += NR_END;
t -= nrl;

t[nrl]=(float **) malloc((size_t)((nrow*ncol+NR_END)*sizeof(float*)));
if (!t[nrl]) nrerror("Allocation error 2 in f3tensor()");
t[nrl] += NR_END;
t[nrl] -= ncl;

t[nrl][ncl]=(float *) malloc((size_t)((nrow*ncol*ndep+NR_END)*sizeof(float)));
if (!t[nrl][ncl]) nrerror("Allocation error 3 in f3tensor()");
t[nrl][ncl] += NR_END;
t[nrl][ncl] -= ndl;

for(j=ncl+1; j<=nch; j++) t[nrl][j]=t[nrl][j-1]+ndep;
for(i=nrl+1; i<=nrh; i++) {
 t[i]=t[i-1]+ncol;
 t[i][ncl]=t[i-1][ncl]+ncol+ndep;
 for(j=ncl+1; j<=nch; j++) t[i][j]=t[i][j-1]+ndep;
 }
return t;
}

/*************** Lower_triangular_matrix *********************************/

int **Lower_triangular_matrix(long nrl, long nrh) {
long i, nrow=nrh-nrl+1;
int  **m;

m=(int **) malloc((size_t)((nrow+NR_END)*sizeof(int *)));
if (!m) nrerror("Allocation error 1 in ltmatrix()");
m += NR_END;
m -= nrl;

m[nrl]=(int *) calloc((unsigned long)(nrow*(nrow+1+NR_END)/2.0), sizeof(int) );
if (!m[nrl]) nrerror("Allocation error 2 in ltmatrix()");
m[nrl] += NR_END;
m[nrl] -= nrl;

for(i=nrl+1; i<=nrh; i++) m[i]=m[i-1]+(i-nrl);
return m;
}
#endif

/*************** Lower_triangular_fmatrix *********************************/

float **Lower_triangular_fmatrix(long nrl, long nrh) {
long i, nrow=nrh-nrl+1;
float **m;

m=(float **) farmalloc((size_t)((nrow+NR_END)*sizeof(float *)));
if (!m) nrerror("Allocation error 1 in ltfmatrix()");
m += NR_END;
m -= nrl;

m[nrl]=(float *) farcalloc((unsigned long)(nrow*(nrow+1+NR_END)/2.0), sizeof(float) );
if (!m[nrl]) nrerror("Allocation error 2 in ltfmatrix()");
m[nrl] += NR_END;
m[nrl] -= nrl;

for(i=nrl+1; i<=nrh; i++) m[i]=m[i-1]+(i-nrl);
return m;
}



void   free_vector(float *v, long nl, long nh) {
free((FREE_ARG) (v+nl-NR_END));
}



void   free_ivector(int  *v, long nl, long nh) {
free((FREE_ARG) (v+nl-NR_END));
}

void   free_matrix(float    **m, long nrl, long nrh, long ncl, long nch) {
free((FREE_ARG) (m[nrl]+ncl-NR_END));
free((FREE_ARG) (m+nrl-NR_END));
}


void   free_cmatrix(char    **m, long nrl, long nrh, long ncl, long nch) {
free((FREE_ARG) (m[nrl]+ncl-NR_END));
free((FREE_ARG) (m+nrl-NR_END));
}

void   free_imatrix(int     **m, long nrl, long nrh, long ncl, long nch) {
free((FREE_ARG) (m[nrl]+ncl-NR_END));
free((FREE_ARG) (m+nrl-NR_END));
}

#if 0
void   free_submatrix(float   **b, long nrl, long nrh, long ncl, long nch) {
free((FREE_ARG) (b+nrl-NR_END));
}

void   free_convert_matrix(float **b, long nrl, long nrh, long ncl, long nch) {
free((FREE_ARG) (b+nrl-NR_END));
}


void   free_f3tensor(float ***t, long nrl, long nrh, long ncl, long nch,
                     long ndl, long ndh) {

free((FREE_ARG) (t[nrl][ncl]+ndl-NR_END));
free((FREE_ARG) (t[nrl]+ncl-NR_END));
free((FREE_ARG) (t+nrl-NR_END));
}

void  Free_ltm(int **m, long nrl, long nrh) {
if (m) {
  free((FREE_ARG) (m[nrl]+nrl-NR_END));
  free((FREE_ARG) (m+nrl-NR_END));
  }
}
#endif

void  Free_ltfm(float **m, long nrl, long nrh) {
if (m) {
  free((FREE_ARG) (m[nrl]+nrl-NR_END));
  free((FREE_ARG) (m+nrl-NR_END));
  }
}




