/*
Virtual migration model simulation.
Ilkka Hanski, Juha Alho and Atte Moilanen.
Numerical implementation by Atte Moilanen 18.03.1997 ->.
*/

#include <stdio.h>
#include <conio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include <stdarg.h>

#include "ran1.h"
#include "kbio.h"
#include "nrutils.h"
#include "fileutil.h"
#include "vmsim.h"
#include "avevar.h"

#define ALPHAVER2 0


extern void Close_rap_file();
extern int Estimate(int gens);

long   idnum;
int    prints=True, rap_mode=True, pri=False;
char   **set_dat;
int    set_rc;

char   data_file_name[80],patch_file_name[80],
       days_file_name[80], output_file_name[80],
       sf_name[80], use_edge, use_bdd, a_or_m;

int    pc, bfc, dayc, mdc, max_surv_int, bfpd, tot_bf;

long   f_evals, bf_tab_size;

float  **dist_mat, **P, **pim, max_area, cap_pm;
int    **b_mat, **ltt, bfd_inp[100], bfd_ip[100], bfd_ep[100], bf_i_ev[100];
int    im_evs, die_evs;
float  **P_vec[365];
float  eta;
float  mu;
float  zeta;
float  beta;
float  delta;
float  B;
float  pat1_cap_mod=1.0; //0.565 for diamina

struct patch     *patches;
struct m_day     *days;
struct butterfly *bflies;

void Mat_cpy(float **m1, float **m2, int cols)
{
  int loop,loop2;
#if 1
  for(loop=0; loop<=cols; loop++)
    memcpy(m2[loop], m1[loop], (cols+1)*sizeof(float));
#else
  for(loop=0; loop<=cols; loop++)
    for(loop2=0; loop2<=cols; loop2++)
      m2[loop][loop2] = m1[loop][loop2];
#endif
}

void Diag(float **m1, int rows)
{
  int loop, loop2;
#if 1
  for(loop=0; loop<=rows; loop++)
    {
       memset(&m1[loop][0], '\0', (rows+1)*sizeof(float));
       m1[loop][loop]=1.0;
    }
#else
  for(loop=0; loop<=rows; loop++)
    for(loop2=0; loop2<=rows; loop2++)
      if (loop!=loop2)
        m1[loop][loop2]=0.0;
      else
        m1[loop][loop2]=1.0;
#endif
}

/***************** Print_matrix ****************************************/
void Print_matrix(char *str, float **m, int rows)
{
  int   loop, loop2;
  float s=0.0;

  Raport("%s\n", str);
  for(loop=0; loop<=rows; loop++)
    {
      s=0.0;
      for(loop2=0; loop2<=rows; loop2++)
        {
          Raport("%-9.6f ", m[loop][loop2]);
          s+=m[loop][loop2];
        }
      Raport(" S:%-10.6f\n", s);
    }
  Raport("\n");
}

/***************** Print_imatrix ****************************************/
void Print_imatrix(char *str, int **m, int rows, int cols)
{
  int   loop, loop2;
  float s=0.0;

  Raport("%s\n", str);
  for(loop=0; loop<=rows; loop++)
    {
      s=0.0;
      for(loop2=0; loop2<=cols; loop2++)
        {
          Raport("%-4i", m[loop][loop2]);
          s+=m[loop][loop2];
        }
      Raport(" S:%0.0f\n", s);
    }
  Raport("\n");
}

/******************** Get_par *****************************************/
int Get_par(char *txt, int term, char *fmt, ...)
{
  int  loop, loop2, found, row;
  va_list  argptr;
  int  cnt;
  char *e_pos, tmp_str[255];

  found = False;
  strlwr(txt);
  for(loop=0; loop<set_rc; loop++)
    if (!strncmp(set_dat[loop], txt, strlen(txt)))
      {
        row  = loop;
        e_pos=strstr(set_dat[row],"=");
        if (e_pos)
          {
            e_pos++;
            while((*e_pos)==' ')
             e_pos++;
            found= True;
            break;
          }
      }

  if (found)
    {
      va_start(argptr, fmt);
      found = vsscanf(e_pos, fmt, argptr);
      va_end(argptr);
      if (found==EOF)
        found=0;
    }

  if (!found)
    {
      if (term)
        {
          clrscr();
          Raport("Could not find compulsory parameter \"%s\" in the settings file,\nterminating.\n", txt);
          Terminate(1,3,"Invalid settings file.");
        }
      else
        {
          Raport("Could not find parameter \"%s\" in the settings file.\n", txt);
          Raport("=> Using default value %s = ", txt);
        }
    }
  else
    {
//      Raport("%s = %s", txt, e_pos);
    }

  return found;
}

void Fid(int *par, int val)
{
  *par = val;
  Raport("%i \n", val);
}

void Ffd(float *par, float val)
{
  *par = val;
  Raport("%0.4f \n", val);
}

int Get_cpar(char *txt, int term, char def_val, char *par)
{
  int cnt;

  cnt = Get_par(txt, term, "%c", par);

  if (cnt==0)
    {
      *par = def_val;
    }

  *par = toupper(*par);
  if (cnt==0)
    {
      if ((*par)==Yes)
        Raport("Yes \n");
      else if ((*par)==No)
        Raport("No \n");
      else
        Raport("%c \n", *par);
    }
  return cnt;
}


/******************** Read_settings ***********************************/
int Read_settings(int ac)
{
  FILE  *settings_file;
  char  in_line[255], ch;
  int   count, s[21];

  clrscr();
  settings_file=Open_file(sf_name, "r+t", "settings", Terminate_on_error);

  set_dat = cmatrix(0,100,0,255);
  set_rc=0;

  while(fgets(in_line, 255, settings_file))
    {
      strncpy(set_dat[set_rc], in_line, 254);
      strlwr(set_dat[set_rc]);
//      printf("%s", set_dat[set_rc]);
      set_rc++;
    }
  fclose(settings_file);

  if (ac<3)
    Get_par("Output file", True, "%s\n", output_file_name);

  remove(output_file_name);

  sprintf(in_line, "copy %s %s", sf_name, output_file_name);
  system(in_line);
  Raport("\n********************************************************************\n");
  Raport("*  Virtual migration model simulation.                             *\n");
  Raport("*  Coded by Atte Moilanen 10.04.1997 ->.                           *\n");
  Raport("*  Dept. Ecology & Systematics, Univ. Helsinki, Finland            *\n");
  Raport("********************************************************************\n\n");

  Get_par("Patch file",  True, "%s\n", patch_file_name);
  Get_par("Data file",   True, "%s\n", data_file_name);
  Get_par("Marking day file", True, "%s\n", days_file_name);

  Get_par("Butterflies per day", True, "%ld\n", &bfpd);
  Get_cpar("Use edgeness", False, No, &use_edge);
  Get_par("Cap. prob. multip.", True, "%f\n", &cap_pm);
  Get_cpar("Add / Multip", False, 'M', &a_or_m);
  Get_cpar("Use birth distr. data", False, No, &use_bdd);

  if (use_edge==Yes)
    Raport("UE\n");
  else
    Raport("!UE\n");

#if ALPHAVER2
  Raport("Using dispersal kernel 1/pow(1+D,alpha)\n");
#else
  Raport("Using dispersal kernel exp(-alpha*D)\n");
#endif


  if (a_or_m=='A')
    Raport("Daily obs. prob. additive\n");
  else if (a_or_m=='M')
    Raport("Daily obs. prob. multiplicative\n");

  Get_par("Mu",   True, "%f\n", &mu);
  Get_par("Eta",  True, "%f\n", &eta);
  Get_par("Zeta", True, "%f\n", &zeta);
  Get_par("Delta",True, "%f\n", &delta);
  Get_par("Beta", True, "%f\n", &beta);
  Get_par("B ",   True, "%f\n", &B);

  Raport(":%-7.3f eta:%-7.3f zeta:%-7.3f :%-7.3f :%-7.3f B:%-7.3f\n",
          mu, eta, zeta, beta, delta, B);

  free_cmatrix(set_dat, 0, 100, 0, 255);
  return True;
}


/******************** Read_patch_data *****************************************/
int Read_patch_data()
{
  FILE  *data_file;
  char  in_line[255];
  int   line=0, temp, loop;

  max_area=1000000.0; tot_bf = 0;
  data_file=Open_file(patch_file_name, "r+t", "input data", Terminate_on_error);
  Raport("READING PATCH DATA: \n");

  while(fgets(in_line, 255, data_file))
    {
      if (sscanf(in_line, "%f %f %f %f\n",
                 &patches[line].x_pos, &patches[line].y_pos,
                 &patches[line].area, &patches[line].pop_size) == 4)
        {

          if (patches[line].area>0.0)
            {
              patches[line].area /= 10000.0;
              if (patches[line].area<max_area)
                max_area = patches[line].area;

              patches[line].pat_num = line+1;
//              Raport("%f %f %f \n", patches[line].x_pos,
//                      patches[line].y_pos, patches[line].area);
              tot_bf += patches[line].pop_size;
              line++;
            }
        }
      else
        {
          fclose(data_file);
          return False;
        }
    }

  pc = line;
  Raport("%i patches read from file\n", pc);
  fclose(data_file);
  return True;
}

/******************** Get_edge_props ************************************/
void Get_edge_props()
{
  int   loop, loop2, pos, loop3;
  float rat, x, y, angle, angles[200], tmp, max_ang;

//  Raport("Edge proportions\n");
  for(loop=0; loop<pc; loop++)
    {
      pos=1;
      for(loop2=0; loop2<pc; loop2++)
        if (loop!=loop2)
          {
            x = patches[loop2].x_pos-patches[loop].x_pos;
            y = patches[loop2].y_pos-patches[loop].y_pos;
            if (x==0)
              {
                if (y<0)
                  angles[pos++] = -M_PI_2;
                else
                  angles[pos++] = M_PI_2;
              }
            else
              angles[pos++]=atan2(y,x);
          }
      for(loop2=1; loop2<pos; loop2++)
        for(loop3=loop2+1; loop3<pos; loop3++)
          if (angles[loop2]>angles[loop3])
            {
              tmp = angles[loop3];
              angles[loop3]  = angles[loop2];
              angles[loop2] = tmp;
            }
      angles[0] = angles[pos-1]-2*M_PI;
      max_ang = 0;
      for(loop2=0; loop2<pos-1; loop2++)
        {
          max_ang = max(max_ang, angles[loop2+1]-angles[loop2]);
//          Raport("E: %-3i %0.4f\n", loop2, angles[loop2]);
        }
      patches[loop].edge_prop = max_ang/(2*M_PI);
//      Raport("%-3i %0.4f\n", loop+1, patches[loop].edge_prop);
    }
}

/******************** Read_day_data *************************************/
int Read_day_data()
{
  FILE  *data_file;
  char  in_line[255];
  int   loop, temp, day_num, places[500], place_count, place, max_bfpd;
  long  row=0;
  float obs_prob, birth_prop, bps=0.0, prev_bp;

  data_file=Open_file(days_file_name, "r+t", "input data", Terminate_on_error);
  Raport("\nREADING CAPTURE PLACE DATA: \n");

  max_bfpd = day_num = place_count=0;

  while(fgets(in_line, 255, data_file))
    {
      row++;
      if ((strlen(in_line)>0) && (in_line[0]!=' '))
        {
          if (in_line[0]=='D')
            {
              if (place_count>0)
                {
                  days[day_num].mpv = calloc(place_count+1, sizeof(int));
//                  days[day_num].ro = 0.6;
                  days[day_num].ro = obs_prob;
                  days[day_num].birth_prop = birth_prop;
                  bps+=birth_prop;
                }
              else
                {
                  days[day_num].mpv = NULL;
                  days[day_num].ro = 0.0;
                  days[day_num].birth_prop = birth_prop;
                  bps+=birth_prop;
                }

              for(loop=0; loop<place_count; loop++)
                days[day_num].mpv[loop]=places[loop];

              days[day_num].mpc=place_count;
              Raport("Day %i, mark-recapture performed in %i patches\n", day_num, days[day_num].mpc);

              sscanf(in_line, "D %i %f %f\n", &day_num, &obs_prob, &birth_prop);
//              max_bfpd = max(max_bfpd, birth_prop*tot_bf);
              place_count = 0;
            }
          else
            {
              sscanf(in_line, "%i\n", &place);
              if (place>pc)
                {
                  Raport("No such patch (%i), row %ld in days file\n",
                          place, row);
                }
              places[place_count++]=place-1;
            }
        }
    }

  if (place_count>0)
    {
      days[day_num].mpv = calloc(place_count+1, sizeof(int));
//      days[day_num].ro  = 0.6;
      days[day_num].ro  = obs_prob;
      days[day_num].birth_prop = birth_prop;
      bps+=birth_prop;
    }
  else
    {
      days[day_num].mpv = NULL;
      days[day_num].ro  = 0.0;
      days[day_num].birth_prop = birth_prop;
      bps+=birth_prop;
    }

  for(loop=0; loop<place_count; loop++)
    days[day_num].mpv[loop]=places[loop];

  days[day_num].mpc=place_count;
  Raport("Day %i, mark-recapture performed in %i patches\n", day_num, days[day_num].mpc);

  dayc = day_num;

  fclose(data_file);

  prev_bp = 0.0;
#if 1
  Raport("\nCumulative birth distribution\n");
  for(loop=1; loop<=dayc; loop++)
    {
      if (days[loop].mpc>0)
        mdc++;
      days[loop].birth_prop = prev_bp +days[loop].birth_prop/bps;
//      Raport("%-2i %0.3f\n", loop, days[loop].birth_prop);
      prev_bp = days[loop].birth_prop;
    }
//  Raport("\n");
#endif
  loop=1;
  do {
      temp=loop;
      do {
        temp++;
      } while ((days[temp].mpc==0) && (temp<dayc));
      days[loop].next_md = temp;
      loop = temp;
  } while (loop<dayc);

  max_surv_int = 0;
  for(loop=1; loop<dayc; loop++)
    if (days[loop].next_md >0)
      max_surv_int = max( days[loop].next_md-loop, max_surv_int);

//  for(loop=1; loop<dayc; loop++)
//    Raport("NMD: %2i %2i \n", loop, days[loop].next_md);
//  Raport("MSI: %i \n\n", max_surv_int);

  return True;
}


/******************** Raport_captures *****************************************/
int Raport_captures()
{
  FILE  *data_file;
  char  in_line[255];
  int   loop,loop2, bfn;

  data_file=Open_file(data_file_name, "w+t", "input data", Terminate_on_error);

  bfn=1;

  for(loop=0; loop<bfc; loop++)
    {
      if (bflies[loop].cpc>0)
        {
          fprintf(data_file, "Butterfly %i  1\n", bfn);
          for(loop2=0; loop2<bflies[loop].cpc; loop2++)
            fprintf(data_file, "%-3i %-3i\n", bflies[loop].ctv[loop2], bflies[loop].cpv[loop2]+1);
          bfn++;
        }
    }

  fclose(data_file);
  return True;
}

/******************** Calculate_distances ********************************/
int Calculate_distances()
{
  int   loop2, loop3;
  float dij;

  printf("CALCULATING DISTANCES: ");

//  dist_mat = Lower_triangular_fmatrix(0, pc-1);
  dist_mat = matrix(0, pc-1, 0, pc-1);
  for(loop2=0; loop2<pc; loop2++)
    for(loop3=0; loop3<pc; loop3++)
      {
        dij= sqrt(pow(patches[loop2].x_pos-patches[loop3].x_pos,2)+
                  pow(patches[loop2].y_pos-patches[loop3].y_pos,2)  );
        dist_mat[loop2][loop3] = dij;
      }

  printf("Ok. Done.\n\n");
  return True;
}


void Get_isols()
{
  int   loop, loop2;
  float d;

  for(loop=0; loop<pc; loop++)
    {
      patches[loop].isolation = 0.0;
      for(loop2=0; loop2<pc; loop2++)
        if (loop!=loop2)
          {
#if ALPHAVER2
            patches[loop].isolation +=
              1.0/(pow(1.0+dist_mat[loop][loop2],beta))*pow(patches[loop2].area, B);
#else
            patches[loop].isolation +=
              exp(-beta*dist_mat[loop][loop2])*pow(patches[loop2].area, B);
#endif
          }
    }

  for(loop=0; loop<pc; loop++)
    {
      d = delta*delta/(delta*delta+pow(patches[loop].isolation,2));
      patches[loop].mu = d;
      patches[loop].migr_prob = eta*pow(patches[loop].area, zeta);
    }
}

void Calc_pim()
{
  int loop, loop2;

  for(loop=0; loop<pc; loop++)
    for(loop2=0; loop2<pc; loop2++)
      if (loop2!=loop)
        {
#if ALPHAVER2
          pim[loop][loop2] =
              1.0/(pow(1.0+dist_mat[loop][loop2],beta))
                  *pow(patches[loop2].area, B)/patches[loop].isolation;
#else
          pim[loop][loop2] =
            exp(-beta*dist_mat[loop][loop2])*pow(patches[loop2].area,B)/
                                             patches[loop].isolation;
#endif
        }
      else
        pim[loop][loop2] = 0.0;
}

/******************** Calc_P *********************************************/
void Calc_P()
{
  int   loop, loop2, loop3;
  float d;

  Get_isols();
  Calc_pim();
  for(loop=0; loop<pc; loop++)
    {
      d = delta*delta/(delta*delta+pow(patches[loop].isolation,2));
      patches[loop].mu = d;
    }

  P[0][0]=1.0;
  for(loop=1; loop<=pc; loop++)
    P[0][loop]=0.0;

  for(loop=1; loop<=pc; loop++)
    P[loop][0] = mu + (1.0-mu)*eta * pow(patches[loop-1].area, zeta)*
                                   patches[loop-1].mu;

  for(loop=1; loop<=pc; loop++)
    for(loop2=1; loop2<=pc; loop2++)
      if (loop2!=loop)
        P[loop][loop2] = (1.0-mu)*eta* pow(patches[loop-1].area, zeta)*
                         (1.0-patches[loop-1].mu)*pim[loop-1][loop2-1];

//  Raport(":%-7.3f eta:%-7.3f zeta:%-7.3f :%-7.3f :%-7.3f B:%-7.3f \n",
//         mu, eta, zeta, beta, delta, B);

  for(loop=1; loop<=pc; loop++)
    {
      P[loop][loop] = (1.0-mu)*(1.0-eta*pow(patches[loop-1].area, zeta));
//      Raport("%f %f \n", patches[loop-1].area,pow(patches[loop-1].area, zeta));
    }

//  for(loop=0; loop<=pc; loop++)
//    for(loop2=0; loop2<=pc; loop2++)
//      P_vec[1][loop][loop2] = P[loop][loop2];

//  Mat_cpy(P, P_vec[1], pc);

//  for(loop=2; loop<=max_surv_int; loop++)
//    Mat_multip(P_vec[loop], P_vec[loop-1], P, pc);
}

void Get_pfpd()
{
  int loop;
  float a_sum=0.0;

  for(loop=0; loop<pc; loop++)
    a_sum += pow(patches[loop].area, B);

  for(loop=0; loop<pc; loop++)
    patches[loop].bfpd = bfpd*pow(patches[loop].area, B)/a_sum;
}

void Gen_init_pop()
{
  int loop, loop2, loop3;

  bfc=0;
  for(loop3=0; loop3<4; loop3++)
    for(loop=0; loop<pc; loop++)
      for(loop2=0; loop2<patches[loop].bfpd; loop2++)
        {
          bflies[bfc].state = loop;
          bflies[bfc].age = 0;
          bflies[bfc].nat_pat = loop;
          bfc++;
        }
}

void Gen_new_bflies()
{
  int loop, loop2;

  for(loop=0; loop<pc; loop++)
    for(loop2=0; loop2<patches[loop].bfpd; loop2++)
      {
        bflies[bfc].state = loop;
        bflies[bfc].age = 0;
        bflies[bfc].nat_pat = loop;
        bfc++;
      }
}

void Gen_new_bflies_distr(int day)
{
  int loop, loop2;

  for(loop=0; loop<pc; loop++)
    for(loop2=0; loop2<b_mat[loop][day]; loop2++)
      {
        bflies[bfc].state = loop;
        bflies[bfc].nat_pat = loop;
        bflies[bfc].age = 0;
        bfc++;
      }
}

void Kill()
{
  int loop;

  for(loop=0; loop<bfc; loop++)
    if (bflies[loop].state>=0)
      {
        if (ran1(&idnum)<mu)
          {
            bflies[loop].state=-10;
            ltt[bflies[loop].nat_pat][bflies[loop].age]++;
          }
      }
}

void Capture(int day)
{
  int loop, loop2, cap, cpc, state;

  for(loop=0; loop<bfc; loop++)
    if ((state=bflies[loop].state)>=0)
      {
        if (ran1(&idnum)<patches[state].ro[day])
          {
            cpc = bflies[loop].cpc;
            bflies[loop].cpv[cpc] = bflies[loop].state;
            bflies[loop].ctv[cpc] = day;
            bflies[loop].cpc++;
          }
      }
}


void Migrate()
{
  int loop, state, loop2;
  float rnum, sum;

  for(loop=0; loop<bfc; loop++)
    if ((state=bflies[loop].state)>=0)
      {
        if (ran1(&idnum)<patches[state].migr_prob)
          {
            if (ran1(&idnum)<patches[state].mu)
              {
                bflies[loop].state = -10;
                die_evs++;
              }
            else
              {
                im_evs++;
                rnum = ran1(&idnum);
                sum  = 0.0;
                for(loop2=0; loop2<pc; loop2++)
                  {
                    sum += pim[state][loop2];
                    if (rnum<=sum)
                      {
                        bflies[loop].state = loop2;
                        bf_i_ev[loop2]++;
                        break;
                      }
                  }
              }
          }
      }
}


void Count_stats()
{
  int loop;

  for(loop=0; loop<bfc; loop++)
    if (bflies[loop].state>=0)
      {
        if (bflies[loop].state==bflies[loop].nat_pat)
          bfd_inp[bflies[loop].state]++;
        else
          {
            bfd_ip[bflies[loop].state]++;
            bfd_ep[bflies[loop].nat_pat]++;
          }
        bflies[loop].age = bflies[loop].age+1;
      }
}

/******************** Simulate **************************************/
void Simulate()
{
  int loop, loop2;

  for(loop=1; loop<=dayc; loop++)
    {
      if (use_bdd==Yes)
        Gen_new_bflies_distr(loop);
      else
        Gen_new_bflies();
      Capture(loop);
      Kill();
      Migrate();
      Count_stats();
      printf("%i \n", bfc);
    }
}

void Fix_cps()
{
  int loop, loop2;

  for(loop=0; loop<pc; loop++)
     for(loop2=0; loop2<=dayc; loop2++)
       patches[loop].ro[loop2] = 0;

  for(loop2=1; loop2<=dayc; loop2++)
    for(loop=0; loop<days[loop2].mpc; loop++)
      {
        if (days[loop2].mpv[loop]==0)
           patches[days[loop2].mpv[loop]].ro[loop2] = days[loop2].ro;
        else
          {
            if (a_or_m=='M')
              patches[days[loop2].mpv[loop]].ro[loop2] = cap_pm*days[loop2].ro;
            else
              patches[days[loop2].mpv[loop]].ro[loop2] = max(days[loop2].ro-cap_pm,0.0);
          }
      }
  Raport("\nCapture prob matrix\n");

  for(loop=0; loop<pc; loop++)
    {
     for(loop2=0; loop2<=dayc; loop2++)
       {
         Raport("%-0.3f ", patches[loop].ro[loop2]);
       }
     Raport("\n");
    }
  Raport("\n");
}

void Fix_b_mat()
{
  int loop, loop2;

  days[0].birth_prop=0.0;

  for(loop=0; loop<pc; loop++)
    {
     b_mat[loop][0]=0;
     for(loop2=1; loop2<=dayc; loop2++)
       {
         b_mat[loop][loop2] = (int)(days[loop2].birth_prop*patches[loop].pop_size)-
                              (int)(days[loop2-1].birth_prop*patches[loop].pop_size);
       }
    }

  Print_imatrix("Individuals being born into each patch daily", b_mat, pc-1, dayc);
}

void Null_stats()
{
  int loop;

  bfc = 0;
  im_evs =0;
  die_evs =0;
  for(loop=0; loop<=pc; loop++)
    {
      bfd_inp[loop] = 0;
      bfd_ep[loop]  = 0;
      bfd_ip[loop]  = 0;
      bf_i_ev[loop] = 0;
    }
}

/******************** Free_data ******************************************/
void Free_data()
{
  int loop;

  Rap();
  free_matrix(dist_mat, 0, pc-1, 0, pc-1);

  for(loop=1; loop<=dayc; loop++)
    free(days[loop].mpv);
  free(days);

  for(loop=0; loop<pc; loop++)
    {
      free(patches[loop].mdv);
      free(patches[loop].captures);
      free(patches[loop].recaps);
      free(patches[loop].ro);
    }
  free(patches);

  for(loop=0; loop<bf_tab_size; loop++)
    {
      free(bflies[loop].cpv);
      free(bflies[loop].ctv);
    }
  free(bflies);
  free_matrix(P,   0, pc+1, 0, pc+1);
  free_matrix(pim, 0, pc+1, 0, pc+1);
  free_imatrix(b_mat, 0, pc+1, 0, dayc+1);
  free_imatrix(ltt, 0, pc+1, 0, dayc+1);

//  for(loop=1; loop<=dayc; loop++)
//    free_matrix(P_vec[loop], 0, pc+1, 0, pc+1);

  Rap();
}

/********************* main *******************************************/
int main(int argc, char **argv)
{
  int   loop, loop2;
  float ave, var;

  strcpy(sf_name, "settings.dat");
  strcpy(output_file_name, "results");
  if (argc>=2)
    strncpy(sf_name,  argv[1], 13);
  if (argc>=3)
    strncpy(output_file_name, argv[2], 13);

  randomize();
  idnum=-1-random(32765);
  ran1(&idnum);

  if (!Read_settings(argc))
    Terminate(1,2, "Invalid settings file -settings.dat");

  if ( !(patches=calloc(300, sizeof(struct patch))) )
    Terminate(1,2, "Error allocating patches");

  if ( !(days=calloc(365, sizeof(struct m_day))) )
    Terminate(1,2, "Error allocating days");

  if (!Read_patch_data())
    Terminate(1,2, "Invalid patch data file ");

  if (!Read_day_data())
    Terminate(1,2, "Invalid day data file ");

  if (!(P = matrix(0, pc+1, 0, pc+1)))
    Terminate(1,2, "Error allocating P ");

  if (!(pim = matrix(0, pc+1, 0, pc+1)))
    Terminate(1,2, "Error allocating pim ");

//  for(loop=1; loop<=dayc; loop++)
//    if (!(P_vec[loop] = matrix(0, pc+1, 0, pc+1)))
//      {
//        Raport("Error allocating p-matrix %i\n", loop);
//        Terminate(1,2, "Error allocating P ");
//      }

  if (!(b_mat = imatrix(0, pc+1, 0, dayc+1)))
    Terminate(1,2, "Error allocating b_mat ");

  if (!(ltt = imatrix(0, pc+1, 0, dayc+1)))
    Terminate(1,2, "Error allocating ltt ");

  for(loop=0; loop<=pc; loop++)
     for(loop2=0; loop2<=dayc; loop2++)
       ltt[loop][loop2]=0;

  for(loop=0; loop<pc; loop++)
    {
      patches[loop].captures = calloc(dayc+1, sizeof(int));
      patches[loop].recaps   = calloc(dayc+1, sizeof(int));
      patches[loop].ro       = calloc(dayc+1, sizeof(float));
    }

  if (!Calculate_distances())
    Terminate(1,2, "Error calculating distances ");

//  Print_matrix("Dist_mat",dist_mat, pc-1);

  Fix_cps();

  Fix_b_mat();

  Get_edge_props();
#if 0
  Raport("Daily capture probabilities:\nday  pat#1 (others)\n");
  for(loop=1; loop<=dayc; loop++)
    {
      if (a_or_m=='M')
        Raport("%-3i %-6.3f (%-6.3f)\n", loop, days[loop].ro, cap_pm*days[loop].ro);
      else
        Raport("%-3i %-6.3f (%-6.3f)\n", loop, days[loop].ro, max(0.0, days[loop].ro-cap_pm));
    }
#endif
  Get_isols();

  Calc_pim();

//  Print_matrix("Pim", pim, pc-1);

  Get_pfpd();

  if (use_bdd==Yes)
    {
      bf_tab_size = tot_bf+2;
    }
  else
      bf_tab_size = (dayc+5)*(long)bfpd;

  if ( !(bflies=calloc(bf_tab_size, sizeof(struct butterfly))) )
    Terminate(1,2, "Error allocating butterflies");

  for(loop=0; loop<bf_tab_size; loop++)
    {
      bflies[loop].cpv = calloc(0.5*dayc, sizeof(int));
      bflies[loop].ctv = calloc(0.5*dayc, sizeof(int));
      if (!bflies[loop].ctv)
        Terminate(1,2,"Out of memory allocating bflies");
      bflies[loop].cpc   = 0;
      bflies[loop].state = -1;
    }

  Raport("\n\nColumn 1: patch#\n");
  Raport("Column 2: isolation of patch\n");
  Raport("Column 3: mortality during migration for animals leaving patch\n");
  Raport("Column 4: migration probability for individual in respective patch\n");
  Raport("Column 5: butterflies born into the patch each day\n");
  Raport("\n#   Isolations, j, mp, bfpd: \n");
  for(loop=0; loop<pc; loop++)
    Raport("%-3i %-9.6f  %-7.4f  %-7.4f  %i\n",
           loop+1, patches[loop].isolation, patches[loop].mu,
           patches[loop].migr_prob, patches[loop].bfpd);
  Raport("\n");

  Null_stats();

  if (use_bdd!=Yes)
    Gen_init_pop();

  Simulate();

  Raport("The following information is based on complete information about \n");
  Raport("of the movements of simulated animals. Note that the capture\n");
  Raport("probabilities given in the day file do not affect this data at all.\n");
  Raport("Capture probabilities in the day file only affect the generation of\n");
  Raport("simulated data that can be used to test the performance of the\n");
  Raport("estimation programs vm2.exe and vmcl2.exe.\n\n");
  Raport("Successfull migration events %i\n", im_evs);
  Raport("Died during migration %i\n", die_evs);
  Raport("\nImmigration events to patches\n");
  for(loop=0; loop<pc; loop++)
    Raport("%-2i  %i\n", loop+1, bf_i_ev[loop]);

  Raport("\n\nButterfly days in:\n");
  Raport("Column 2:  natal_patch\n");
  Raport("Column 3:  immigrated to patch\n");
  Raport("Column 4:  emigrated from patch\n");
  for(loop=0; loop<pc; loop++)
    Raport("%-2i  %-4i  %-4i  %-4i\n", loop+1, bfd_inp[loop], bfd_ip[loop], bfd_ep[loop]);

  Raport_captures();

  Raport("\n\nDistribution of lifetimes in the patch network\n");
  Raport("for individualss originating from different patches (1st col., 0 days)\n");
  Print_imatrix("", ltt, pc, dayc);


  Free_data();

  if (!rap_mode)
    Close_rap_file();

  fcloseall();

  return EXIT_SUCCESS;
}

